/* $Id: client.c,v 1.22 1998/05/24 23:35:48 majdi Exp $ */
#include "config.h"

/* pre-defines */
char *myname, *type_to_name(int type);
int facility = -2, dnslog = 0, mallocdebug = 0, dnsexpire = 0, debug = 0;
struct dnscache *dnshead = NULL;
struct clientstatus *clienthead = NULL;
struct downdata {
	char hostname[256];
	int type;
	int port;
	int lastcheck;
	int downct;
	int notified;
	time_t deathtime;
	} ;

void	do_exit()
{
	/* set terminal back */
#ifdef NICEINTERFACE
	nocbreak();
	echo();
	nl();
#endif	/* NICEINTERFACE */
	/* done setting terminal back */
	exit(0);
	/* never reached */
}

void	client_poll(int local_timeout)
{
	sleep(local_timeout);
}

void	parse_data(char *buff, struct downdata *stuff, int bufflen)
{
	int x = 0, end, start = 0, field = 0;
	char space[256];

	/*
	** Rather than go through a bunch of sequential loops, we
	** might as well make this one big loop and keep track of
	** which field we're parsing. --Majdi
	*/

	for (;x < bufflen;x++)
	{
		if (buff[x] == ':')
		{
			field++;
			end = x;

			memset(space,0,sizeof(space));

			strncat(space, buff+start, (end - start));

			switch(field)
			{
				case 1:
					strcpy(stuff->hostname, space);
					break;
				case 2:
					stuff->type = atoi(space);
					break;
				case 3:
					stuff->port = atoi(space);
					break;
				case 4:
					stuff->lastcheck = atoi(space);
					break;
				case 5:
					stuff->downct = atoi(space);
					break;
				case 6:
					stuff->notified = atoi(space);
					break;
				case 7:
					stuff->deathtime = atol(space);
					break;
			}
			start = x+1;
		}
	}
}

void	show_data(struct downdata down)
{
	char *data = timedata(down.deathtime);
#ifdef NICEINTERFACE
	printw("%-25.24s%-6s%-5d%-6d%-6s%-15s%s\n", down.hostname,
                type_to_name(down.type), down.port, down.downct,
                yes_no(down.notified), errtostr(down.lastcheck), data);
#else
	fprintf(stdout,"%-25s%-6s%-5d%-6d%-6s%-15s%s\n", down.hostname,
                type_to_name(down.type), down.port, down.downct,
                yes_no(down.notified), errtostr(down.lastcheck), data);
#endif /* NICEINTERFACE */
        FREE(data);
}

#ifdef NICEINTERFACE
void	bottom_banner(WINDOW *wdw)
{
	int ctr, maxx, maxy;

	getmaxyx( wdw, maxy, maxx); /* get the max x, and the max y */

	setsyx( maxy-3, 0);
        for ( ctr = 0; ctr < maxx ; ctr++)
                printw("-");
	printw(" q = quit   space = refresh   h = help\n");
}

#endif /* NICEINTERFACE */

#ifdef NICEINTERFACE
void	top_banner(char *server, WINDOW *wdw)
#else
void	top_banner(char *server)
#endif
{
        time_t t;
        char nfo[30];

#ifdef NICEINTERFACE
	int maxx, maxy;

	getmaxyx( wdw, maxy, maxx); /* get the max x, and the max y */
#endif

        time (&t); /* get the time */
        strcpy(nfo,ctime(&t)+4); /* convert it to a string, copy to buffer */

#ifdef NICEINTERFACE
	setsyx(0,0); /* ok, put it at the top */
	printw("Server: %-30s%-20s%-20s", server, "     Current Time: ", nfo);
	printw("%-25s%-6s%-5s%-6s%-6s%-15s%s\n", "Hostname", "Type", "Port", 
		"Count", "Notif", "Stat", "Time Failed");
	for (t = 0;t < maxx;t++)
		printw("-");
	doupdate();
	refresh();
#else
	fprintf(stdout,"%-25s%-6s%-5s%-6s%-6s%-15s%s\n", "Hostname", "Type", 
		"Port", "Count", "Notified", "Stat", "Time Failed");
#endif
}

void	text_query(char *server, int port)
{
}

#ifdef NICEINTERFACE
void	start_client(char *server, int port, WINDOW *wdw)
#else
void	start_client(char *server, int port)
#endif
{
	int ret, filedes;
	char buff[256];
	struct downdata data;
	struct my_hostent *hp;

	if (! (hp = my_gethostbyname(server)) )
	{
		fprintf(stderr, "unable to find %s in DNS\n", server);
		return;
	}


	/* Do the open_host call */
	ret = open_host(server, port, &filedes, 20);
	if (ret == SYSM_INPROG)
	{
		if (debug)
		{
			printf("Connection in progress, waiting until FD is ready\n");
		}
		blocktillready(filedes, 20);
		
	}

	if (debug)
		fprintf(stdout,"open_host returned %d with fd %d\n", ret, 
			filedes);
	if (ret != 0)
	{
		fprintf(stderr, "Unable to connect to %s:%d\n", server, port);
		exit(1);
	}
	
	getline_tcp(filedes, buff);
	if (debug)
		fprintf(stdout,"Got this: %s\n", buff);
	if (strncmp(buff, "111", 3) != 0)
	{
		fprintf(stderr,"invalid response from server\n");
		fprintf(stderr,"got:%s:\n", buff);
		do_exit();
	}
	if (debug)
		fprintf(stdout,"sending status request\n");
	sendline(filedes, "STAT");
#ifdef NICEINTERFACE
	top_banner(server, wdw);
#else
	top_banner(server);
#endif
	while (1)
	{
		getline_tcp(filedes, buff);
		if (debug)
			fprintf(stdout,"got this: :%s:\n", buff);
		/* hostname, type, port, lastcheck, downct, deathtime */
		if (strncmp(buff, "333", 3) == 0)
			break; /* done reading data */
		/* parse the line to extract the important data */
		parse_data(buff, &data, sizeof(buff));
		show_data(data);
	}
#ifdef NICEINTERFACE
	clrtobot();
	bottom_banner(wdw);
#endif
	
	sendline(filedes, "QUIT");
	getline_tcp(filedes, buff);
	if (close(filedes) == -1)
		return;
}

#ifdef NICEINTERFACE
void	my_client_sleep(int sleeptime, WINDOW *wdw)
{
        time_t start = time(NULL); /* record start time */
        time_t now = time(NULL); /* time now */
        char ch = '\0', nfo[30];
        int maxx, maxy;

        getmaxyx( wdw, maxy, maxx); /* get the max x, and the max y */
	doupdate();
	refresh();

	if (debug)
	        fprintf(stdout, " starting to sleep... \n");

        while ((now-start) < sleeptime)
        {
		/* 0 = stdin */
		if (data_waiting_read(0, 1))
		{
			ch = getch();
			if (ch == 'q' || ch == 'Q')
			{
				printw("\n");
				doupdate();
				refresh();
				do_exit();
			} 
			else
				return;
		/* normally we would parse the input and do other
		   fancier things, but not now */
		}

	/* should really be doing a select() on stdin with a timeout value
		of one second */

                time(&now);
		strcpy(nfo,ctime(&now)+4); /* convert it to a string */
		setsyx(0, 58);
		doupdate();
		refresh();
		printw("%s", nfo);
		doupdate();
		refresh();
		setsyx(maxy-1, 0);
		doupdate();
		refresh();

        }
	if (debug)
	        fprintf(stdout, " waking up \n");

}

#endif /* NICEINTERFACE */

int	main(int argc, char **argv)
{
	char server[256]; /* remove server to connect to */
	int port = PORTNUM; /* remote port number */
#ifdef NICEINTERFACE
	WINDOW *wdw;
	int maxx, maxy;
#endif /* NICEINTERFACE */

	myname = argv[0]; /* save myname */

	switch(argc)
	{
		case 2:
			strcpy(server, argv[1]);
			break;
		case 3:
			strcpy(server, argv[1]);
			port = atoi(argv[2]);
			break;
		default:

			fprintf(stderr,"usage: %s server-host-name [ port ]\n", 
				argv[0]);
			exit(1);
	}

	if (debug)
		fprintf(stderr, "host: %s port: %d \n", server, port);

#ifdef NICEINTERFACE
	/* ncurses setup stuff */
	initscr();
	cbreak();
	noecho();
	nonl();
	intrflush(stdscr, FALSE);
	keypad(stdscr, TRUE);
	/* end ncurses setup */

	clear();
	wdw = newwin(0,0,0,0); /* full screen window */
	nodelay(wdw, TRUE);
	printw("Connecting to server %s and getting inital data...\n", server);
	/* ncurses update the screen stuff */
	doupdate();
	refresh();
	clear(); /* for when the next screen refresh happens */
	start_client(server, port, wdw);
#else
	start_client(server, port);
	return(0);
#endif

#ifdef NICEINTERFACE
	while (1)
	{
		getmaxyx(wdw, maxy, maxx);
		doupdate();
		refresh();

		my_client_sleep (60, wdw);

		start_client(server, port, wdw);
	}
#endif /* NICEINTERFACE */
	return(0);
}
