/* $Id: config.h.tmpl,v 1.95 1998/01/26 01:57:20 jared Exp $ */

#include "defines.h"

#ifdef HAVE_LIBNCURSES
#define NICEINTERFACE  /* define for ncurses interface in client */
#endif /* HAVE_LIBNCURSES */

#include <signal.h>
#include <stdio.h> 
#include <ctype.h>
#include <stdlib.h>
#include <time.h>  
#include <netdb.h>
#include <string.h>
#include <pwd.h>
#ifdef __svr4__   /* slo-laris */
#include <sys/filio.h>
#endif
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/stat.h>
#ifdef _AIX32
#include <fcntl.h>
#else /* _AIX32 */
#include <sys/fcntl.h>
#endif /* _AIX32 */
#include <errno.h>
#include <unistd.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#include <netinet/ip_icmp.h>
#include <arpa/inet.h>
#include <syslog.h>
#ifdef __svr4__
#include <stdarg.h>
#include <varargs.h>
#endif
#ifdef NICEINTERFACE
#ifdef sgi
#include "/usr/local/include/ncurses.h"
#else
#include <ncurses.h>
#endif
#endif
#include <sys/utsname.h>


#define VERS            "System Monitor version v0.80"
#define CFILE           "/etc/sysmon.conf"
#define PMESG           "%t: %h %w is %u %d"
#define PORTNUM		1345
#define	MAX_ARGS	100

/* the following should be read from /etc/services */

/* Per rfc */
#define SMTP_PORTNUM 25

/* per rfc */
#define HTTP_PORTNUM 80

/* per rfc */
#define IMAP_PORTNUM 143

/* per rfc */
#define NNTP_PORTNUM 119

/* per rfc? */
#define POP2_PORTNUM 109

/* per rfc? */
#define POP3_PORTNUM 110

/* */
#define DNS_PORTNUM 53

/* Radius */
#define RADIUS_PORTNUM 1643

/* SSL Http */
#define HTTPS_PORTNUM 443

/* Bootp */
#define BOOTP_CLIENT 68
#define BOOTP_SERVER 67

#ifndef bool
#define bool unsigned char
#endif

#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE 1
#endif

/* Check Types */
#define SYSM_TYPE_TCP	1 /* TCP Checks */
#define	SYSM_TYPE_UDP	2 /* UDP Checks */
#define SYSM_TYPE_PING	3 /* Ping */
#define	SYSM_TYPE_SNMP	4 /* Snmp based checks */
#define	SYSM_TYPE_NNTP	5 /* nntp checks */
#define SYSM_TYPE_SMTP	6 /* smtp checks */
#define SYSM_TYPE_IMAP	7 /* imap check */
#define	SYSM_TYPE_POP3	8 /* pop3 check */
#define SYSM_TYPE_X500	9 /* umichX500 check */
#define SYSM_TYPE_POP2	10 /* pop2 check */
#define SYSM_TYPE_BOOTP	11 /* bootp check */
#define	SYSM_TYPE_DNS	12 /* check dns server */
#define SYSM_TYPE_WWW	13 /* www content check */
#define SYSM_TYPE_RADIUS 14 /* radius server check */
#define SYSM_TYPE_HTTPS	15 /* https content check */
#define SYSM_TYPE_SYSM	16 /* check another sysmond */
#define SYSM_TYPE_SSHD	17 /* sshd check */

/* Return Values */
#define SYSM_OK 	0
#define SYSM_CONNREF 	1
#define SYSM_NETUNRCH 	2
#define SYSM_HOSTDOWN	3
#define SYSM_TIMEDOUT 	4
#define SYSM_NODNS	5
#define SYSM_UNPINGABLE	6
#define SYSM_THROTTLED	7
#define SYSM_NOAUTH	8
#define SYSM_NORESP	9
#define SYSM_INPROG	10
#define SYSM_BAD_AUTH 	11
#define SYSM_BAD_RESP 	12
#define X500_WEDGED 	13
#define SYSM_KILLED	14 /* killed locally */

struct hostinfo {
        unsigned char hostname[50]; /* name of system to check */
        unsigned int type; /* 1 = tcp, 2 = udp, 3 = ping, 4 = snmp, 5 = nntp 
		6 = smtp, 7 = imap, 8 = pop3 9 = umichX500 10 = pop2
		11 = bootp 12 = dns 13 = www-content */
        unsigned int port; /* only relevant for tcp/udp */
        unsigned char message[100]; /* message to print for outages */
        unsigned char contact[100]; /* e-mail contact for this */
	unsigned char snmp_community[50]; /* snmp community */
        unsigned int lastcheck; /* lastchecked status  0 = ok,  1 = conn ref
		2 = ENETUNREACH, 3 = EHOSTDOWN||EHOSTUNREACH,
		4 = ETIMEDOUT, 5 = no dns entry, 6 = unpingable,
		7 = Throttled, 8 = no auth, 9 = no response 
		10 = connection in progress, 11 = bad auth 
		12 = bad remote response, 13 = x500 error */
	unsigned char username[50]; /* for pop3, imap, etc */
	unsigned char password[50]; /* for pop3, imap, etc */
	unsigned char url[80]; /* url for www */
	unsigned char url_text[80]; /* text to find within url */
	unsigned int downct; /* number of times counted as down.. */
	unsigned int max_down; /* max times down before we contact someone */

	bool contacted; /* true if mailed contact -- false if not */
	time_t lastcontacted; /* last time we paged someone */
	bool contact_when_up; /* page someone when it comes up */
	
	char queued; /* 0 if not, 1 if in queue */

	long int inaddr; /* for icmp.c */

	time_t lchecktime; /* time last checked */
	time_t check_start; /* time of start of check */
	time_t deathtime; /* time of death ;-) */
	time_t last_up; /* time it last came back */

        struct hostinfo *child, *sibling;
                /* child is any hosts behind this router
                   or any services that this host provides
                   (except for ping) */
                /* sibling is any hosts on this network */
        } ;

/* for lib.c:parse() */
struct parsed {
	int count;
	char **data;
	} ;

/*  */
struct monitorent {
	struct hostinfo *checkent;
        struct monitorent *next;
	struct timeval queueat; /* time we got queued at */
	struct timeval lastserv; /* time we got last serviced */
	short int started; /* is the check actually started yet */
	short int retval; /* set to the return val, or -1 if check not
			done yet */
	void *monitordata; /* should be free'ed when retval is set */
	};

/* client status */
struct clientstatus {
	time_t lastactivity;
	short int filedes;
	char un[80]; /* username */
	char ip[16];
	int authlvl;
	int clientver;
	char buffer[256];
	struct clientstatus *next;
	};

/* my version of struct hostent -- used in dnscache.c */
struct my_hostent {
  char *h_name;			/* Official name of host */
  int h_addrtype;               /* Host address type.  */
  int h_length;                 /* Length of address.  */
  char *my_h_addr;           /* List of addresses from name server.  */
};


/* the actual dns cache list */
struct dnscache {
	char hostname[80];
	struct my_hostent *hp;
	time_t lastquery;
	
	struct dnscache *next;
	};

/* we should use this someday */
struct bootp_pkt {
        unsigned char   bp_op;          /* packet opcode type */
        unsigned char   bp_htype;       /* hardware addr type */
        unsigned char   bp_hlen;        /* hardware addr length */
        unsigned char   bp_hops;        /* gateway hops */
        unsigned long	bp_xid;         /* transaction ID */
        unsigned short  bp_secs;        /* seconds since boot began */
        unsigned short  bp_unused;
        struct in_addr  bp_ciaddr;      /* client IP address */
        struct in_addr  bp_yiaddr;      /* 'your' IP address */
        struct in_addr  bp_siaddr;      /* server IP address */
        struct in_addr  bp_giaddr;      /* gateway IP address */
        unsigned char   bp_chaddr[16];  /* client hardware address */
        unsigned char   bp_sname[64];   /* server host name */
        unsigned char   bp_file[128];   /* boot file name */
        unsigned char   bp_vend[64];    /* vendor-specific area */
};




#define BOOTP_REQUEST 1
#define BOOTP_REPLY 2

/* Maximum times to try if you get a connection refused */
#define MAX_TRIES       5

/* ****************************************** *
 * NO USER SERVICABLE PARTS BEYOND THIS POINT *
 * ****************************************** */

#define HEARTBEAT_HOST "puck.nether.net"
#define HEARTBEAT_PORT 1345


/* misc defines for any/all external functions */
extern int errno;
extern char *myname; /* my called name when I startup */

/* new routine names */

void start_test_dns(struct monitorent *);
void service_test_dns(struct monitorent *);
void start_test_bootp(struct monitorent *);
void service_test_bootp(struct monitorent *);
void start_test_www(struct monitorent *);
void service_test_www(struct monitorent *);
void start_test_https(struct monitorent *);
void service_test_https(struct monitorent *);
void start_test_ping(struct monitorent *);
void service_test_ping(struct monitorent *);
void start_test_imap(struct monitorent *);
void service_test_imap(struct monitorent *);
void start_test_nntp(struct monitorent *);
void service_test_nntp(struct monitorent *);
void start_test_pop3(struct monitorent *);
void service_test_pop3(struct monitorent *);
void start_test_radius(struct monitorent *);
void service_test_radius(struct monitorent *);
void start_test_smtp(struct monitorent *);
void service_test_smtp(struct monitorent *);
void start_test_tcp(struct monitorent *);
void service_test_tcp(struct monitorent *);
void start_test_udp(struct monitorent *);
void service_test_udp(struct monitorent *);
void start_test_x500(struct monitorent *);
void service_test_x500(struct monitorent *);
void stop_test_tcp(struct monitorent *);
void stop_test_udp(struct monitorent *);
void stop_test_ping(struct monitorent *);
void stop_test_snmp(struct monitorent *);
void stop_test_nntp(struct monitorent *);
void stop_test_smtp(struct monitorent *);
void stop_test_imap(struct monitorent *);
void stop_test_pop3(struct monitorent *);
void stop_test_x500(struct monitorent *);
void stop_test_pop2(struct monitorent *);
void stop_test_bootp(struct monitorent *);
void stop_test_dns(struct monitorent *);
void stop_test_www(struct monitorent *);
void stop_test_radius(struct monitorent *);
void stop_test_https(struct monitorent *);
void stop_test_sysm(struct monitorent *);
void stop_test_sshd(struct monitorent *);
int is_open(int);
void free_tree(struct hostinfo *);
void stop_daemon();
void print_queue(int );
void reload_config();
struct hostinfo *sync_after_sighup(struct hostinfo *, char *);
float mydifftime(struct timeval, struct timeval);
void blocktillready(int, int);
void expire_dns();
void nop(); /* a do nothing function... */
void check_runtime(time_t, time_t, char *); /* Logs funcs that suck time */
int check_smtp(char*);
int test_imap(char *, char *, char *);
int um_x500_monitor(char *, int);
void do_tree_periodic();
int tcp_open_sock(int);
int nextfd();
int udp_open_sock();
int init_socket();
int icmp_open_sock(int);
struct my_hostent *my_gethostbyname(char *);
void client_poll();
int test_udp(char*, int);
int test_tcp(char*, int);
int sendline(int , char *);
void hard_copy(struct hostinfo *old, struct hostinfo *new);
void dump_to_file(char *, int );
void add_line(FILE *fh, struct hostinfo down, int html);
int data_waiting_read(int, int);
void walk_this_way(FILE *fh, struct hostinfo *here, int html);
int check_nntp(char *);
void timeout_clients();
void dead_client_cleanup();
int ping(struct hostinfo *);
char *str_difftime(time_t );
char *str_difftime_sec(time_t );
void page_someone(struct hostinfo*);
struct hostinfo *parseline(int *, int*, char*, FILE*);
int test_pop3(char*,char*,char*);
int check_http(char*,char*, char*);
char *yes_no(int);
int getline_tcp(int , char *);
int can_write(int, int);
struct hostinfo *loadconfig(char*);
int parse(char *, struct parsed *);
void free_parsed(struct parsed *);
int open_host(char*, int, int*, int);
int open_sock();
char *errtostr(int);
char *type_to_name(int);
char *timedata(time_t);
void syslogmsg(char *);
void send_heartbeat(char *);

/* auth dns check */
int gethost(char *,struct sockaddr_in *);
int check_authdns(char *,char *);
int chk_ns(char *, char *,int,int ,int , int ,int );

/* radius check */
int check_radius(char *host,char *user,char *password,char *secret);
int queryServer(char *hostname, char *username, char *password, char *secret);
void md5_calc (unsigned char *, unsigned char *, unsigned int);

/* https */

int check_https(char *host,char *url,char *txt,int port);


void *MALLOC(size_t);
void FREE(void *);

#ifdef  NICEINTERFACE
void pretty_print_down(struct hostinfo*);
void update_screen();
void setup_screen();
#endif

#ifdef linux
#define ICMPHDR icmphdr
#define ICMP_TYPE type
#define ICMP_CHECKSUM checksum
#define ICMP_CODE code
#define IPHDR iphdr
#define IHL ihl
#define ICMP_SEQ un.echo.sequence
#define ICMP_ECHO_ID un.echo.id
#ifndef RLIMIT_OFILE   
#define RLIMIT_OFILE RLIMIT_NOFILE
#endif            
#endif

#if (defined(__NetBSD__) || defined(__FreeBSD) || defined(sgi)|| \
	defined(FreeBSD) || defined(__FreeBSD__) || defined(__bsdi__))

#ifdef sgi
#define RLIMIT_OFILE RLIMIT_NOFILE
#endif
#define ICMPHDR icmp
#define ICMP_CHECKSUM icmp_cksum
#define ICMP_TYPE icmp_type
#define ICMP_CODE icmp_code
#define IPHDR ip
#define IHL ip_hl
#define ICMP_SEQ icmp_seq
#define ICMP_ECHO_ID icmp_id
#endif

#if (defined(sun) && defined(unix))   /* sunos, solaris, should work for most all */
#define ICMPHDR icmp
#define ICMP_CHECKSUM icmp_cksum
#define ICMP_TYPE icmp_type
#define ICMP_CODE icmp_code
#define IPHDR ip
#define IHL ip_hl
#define ICMP_SEQ icmp_seq
#define ICMP_ECHO_ID icmp_id
#endif

/* need better checks for alpha -- broke on a netbsd alpha box
 * this came from an OSF/1 machine
#ifdef __alpha__
#define ICMPHDR icmp
#define ICMP_CHECKSUM icmp_cksum
#define ICMP_TYPE icmp_type
#define ICMP_CODE icmp_code
#define IPHDR ip
#define IHL ip_vhl
#define ICMP_SEQ icmp_seq
#define ICMP_ECHO_ID icmp_id
#endif
 */

#ifdef _AIX /* AIX */
#define ICMPHDR icmp
#define ICMP_CHECKSUM icmp_cksum
#define ICMP_TYPE icmp_type
#define ICMP_CODE icmp_code
#define IPHDR ip
#define IHL ip_vhl
#define ICMP_SEQ icmp_seq
#define ICMP_ECHO_ID icmp_id
#endif


#ifndef hpux
#ifdef _HPUX_SOURCE
#define hpux
#endif /* _HPUX_SOURCE */
#endif /* hpux */

#ifdef hpux
#define ICMPHDR icmp
#define ICMP_CHECKSUM icmp_cksum
#define ICMP_TYPE icmp_type
#define ICMP_CODE icmp_code
#define IPHDR ip
#define IHL ip_hl
#define ICMP_SEQ icmp_seq
#define ICMP_ECHO_ID icmp_id
#endif

