/* $Id: dnscache.c,v 1.35 1997/12/12 18:00:11 jared Exp $ */

#include "config.h"

extern struct dnscache *dnshead;
extern int dnslog, dnsexpire, debug;

int cache_misses = 0, cache_hits = 0;
int dnsentries = 0;
time_t last_log = 0;

void	expire_dns()
{
	struct dnscache *here = NULL;
	struct dnscache *last = NULL, *freethis = NULL;
	time_t now = 0;
	char buffer[256];
	int expired = 0;
	int count = 0;
	
	/* Check for a possible error case */
	if (dnshead == NULL)
		return;

	/* Count the elements in the dns cache */
	for (here=dnshead; here != NULL; here = here->next)
	{
		count++;
	}

	/* log the number of elements */
	if (debug)
	{
		snprintf(buffer, 254, "counted dns elements = %d", count);
		syslogmsg(buffer);
	}
	
	/* Save the current time */
	time(&now);
	
	here = dnshead;

	while (here != NULL)
	{
		if (freethis != NULL)
		{
			if (freethis->hp != NULL)
			{
				/* free ->hp->h_name */
				FREE(freethis->hp->h_name);
				freethis->hp->h_name = NULL;

				/* free ->hp_>my_h_addr */
				FREE(freethis->hp->my_h_addr);
				freethis->hp->my_h_addr = NULL;
			
				/* free ->hp */
				FREE(freethis->hp);
				freethis->hp = NULL;

				/* free * */
				FREE(freethis);
			}

			/* Count number of expired elements */
			expired++;

			/* decrement entries in the dns list accordingly */
			dnsentries--;

			/* set the fact we've got nothing to free() since
			   we just did it */

			freethis = NULL;
		}

		if ((now- here->lastquery) > (dnsexpire))
		{
			/* entry needs expiry */
			if (last == NULL)	/* set the new dnshead */
			{
				if (dnshead != NULL)
				{
					dnshead = here->next;
				}
			}
			else 
			{
				last->next = here->next;
				/* set the previous entry to point to the next
				   one so we can unlink the current entry */
			}

			/* tag the data for freeing */
			freethis = here;
		} 
		else 
		{
			last = here;
		}
		here = here->next;
	}

	if (freethis != NULL)
	{
		if (freethis->hp != NULL)
		{
			/* free ->hp->h_name */
			FREE(freethis->hp->h_name);
			freethis->hp->h_name = NULL;

			/* free ->hp_>my_h_addr */
			FREE(freethis->hp->my_h_addr);
			freethis->hp->my_h_addr = NULL;

			/* free ->hp */
			FREE(freethis->hp);
			freethis->hp = NULL;
		}

		/* free * */
		FREE(freethis);
		/* decrement entries in the dns list accordingly */
		dnsentries--;

		expired++;
		freethis = NULL;
	}

	/* log periodic data about expiration etc.. */
	if (last_log == 0)
	{
		/* Don't log when we just booted */
		last_log = now;
		return;
	}

	if ((now - last_log ) > (dnslog))
	{
		/* generate stats string */
		snprintf(buffer, 254,
			"dnscache - %d hit %d miss %d exp %d count entr %d est entries",
			cache_hits, cache_misses, expired, count, dnsentries);
		/* Log it */
		syslogmsg(buffer);

		last_log = now;
	}
}
	
struct my_hostent	*add_dns_cache(char *hostname)
{
	struct dnscache *newent;
	struct hostent hp, *hp2 = gethostbyname(hostname);

	if (!hp2)
		return NULL;

	memset(&hp, 0, sizeof (struct hostent));
	memcpy(&hp, hp2, sizeof(struct hostent));

	if (!&hp)
		return NULL;

	/* allocate memory for the new entry */
	newent = MALLOC(sizeof(struct dnscache));

	memset(newent, 0, sizeof(struct dnscache));

	/* copy hostname over */
	strcpy(newent->hostname, hostname);

	/* allocate memory for this entry */
	newent->hp = MALLOC(sizeof(struct my_hostent));
	
	/* save the current time */
	time(&newent->lastquery);

	/* copy needed date to new hp thingie */

	newent->hp->h_name = strdup(hp.h_name);
	newent->hp->h_addrtype = hp.h_addrtype;
	newent->hp->h_length = hp.h_length;

	newent->hp->my_h_addr = MALLOC(hp.h_length);
	memset(newent->hp->my_h_addr, 0, hp.h_length);

	memcpy(newent->hp->my_h_addr, hp.h_addr, hp.h_length);

	newent->next = dnshead;
	dnshead = newent;
	cache_misses++;
	dnsentries++;
	return newent->hp;
}

struct my_hostent *my_gethostbyname(char *hostname)
{
	struct dnscache *here;
	char buffer[128];
	time_t start=time(NULL);

	if (debug)
	{
		snprintf(buffer, 127, "Estimated number of entries in dns cache is %d", dnsentries);
		syslogmsg(buffer);
	}

	expire_dns();

	for (here = dnshead; here != NULL; here = here->next)
	{
		if (strcmp(hostname, here->hostname) == 0)
		{
			cache_hits++;
	                if (time(NULL)-start >5)
	                        abort();
			return here->hp;
		}
	}

        if (time(NULL)-start >5)
                abort();

	return add_dns_cache(hostname);
}
