/* $Id: http.c,v 1.15 1997/11/18 16:13:35 jared Exp $ */
#include "config.h"

extern int debug;
/* returns status based upon this:

  0 = ok,  1 = conn ref, 2 = ENETUNREACH, 3 = EHOSTDOWN||EHOSTUNREACH,
  4 = ETIMEDOUT, 5 = no dns entry, 6 = unpingable, 
  7 = Throttled, 8 = no auth, 9 = no response
  12 = bad response
 */

struct wwwdata {
        int filedes;
	int timesblank;
        time_t start; /* test start time */
        time_t lastact; /* last activity */
        int state;
        };

#define WWW_CONNECT		1 /* doing a connect() */
#define WWW_SENT_REQUEST	2 /* Waiting for the banner */

void	start_test_www(struct monitorent *here)
{
	/* start the test, get connect() rolling and let service_test_www()
	   take care of the rest, return either the tcp errorcode, or
	   that the text was not found 
	 */
        struct wwwdata *localstruct = NULL;
        struct my_hostent *hp = NULL;
        struct sockaddr_in name;
        int errcode = 0;

        /* Allocate our memory */
        here->monitordata = MALLOC(sizeof(struct wwwdata));

        localstruct = here->monitordata;

        time(&localstruct->start);

        localstruct->lastact = localstruct->start;

        localstruct->filedes = open_sock();

        if (localstruct->filedes == -1)
		abort();	/* Bad error, code around it later */

        hp = my_gethostbyname(here->checkent->hostname);

        /* zero out the space */
        memset ( &name, 0, sizeof ( name ) );

        /* copy data */
        memcpy((char*)&name.sin_addr, (char*)hp->my_h_addr, hp->h_length);

        /* set family type */
        name.sin_family = AF_INET;

        /* set the port we're connecting to */
        name.sin_port = htons(here->checkent->port);

        if (debug)
                printf("start_test_www() doing connect() to %s:%d\n",
                        here->checkent->hostname, here->checkent->port);

        errcode = connect(localstruct->filedes, (struct sockaddr*)&name,
                sizeof(struct sockaddr_in));

        if ((errcode < 0) && (errno != EINPROGRESS))
        {
                if (close(localstruct->filedes) == -1)
                        abort(); /* close the fd incase it's left */
		switch (errno)
		{
			case ECONNREFUSED:
			case EINTR:
				here->retval = SYSM_CONNREF;
				break;
			case ENETUNREACH:
				here->retval = SYSM_NETUNRCH;
				break;
			case EHOSTDOWN:
			case EHOSTUNREACH:
				here->retval = SYSM_HOSTDOWN;
				break;
			case ETIMEDOUT:
                        	here->retval = SYSM_TIMEDOUT; 
				break;
			default:
				break;
                }

                /* Free memory we'd normally leak */
                FREE(localstruct);
                here->monitordata = NULL;

                return;
        }

        /* doing a connect() */
        localstruct->state = WWW_CONNECT;

        /* poll it again later */
        return;
}

void	service_test_www(struct monitorent *here)
{
	/* take the filedes created in start_test_www() and watch for it to
	   be writable.  If so, send the http request to retrieve the page.
	   If the text is found, we're in good shape, otherwise, return the
	   appropriate error code
	 */

        struct wwwdata *localstruct = NULL;
        char buffer[1024];
        int x, isopenretval = -1;

        /* do some variable shufflign */
        localstruct = here->monitordata;
	if (localstruct == NULL)
	{
		printf("bug! - localstruct == NULL in http.c:service_test_www\n");
		return;

	}

        /* do different things based on the state */
        switch (localstruct->state)
        {
                case WWW_CONNECT:
                {
                        isopenretval = is_open(localstruct->filedes);
                        if ((isopenretval != -1) &&
                                (isopenretval != SYSM_INPROG))
                        {
                                time(&localstruct->lastact);
                                if (debug)
                                        printf("is_open() in service_test_http()\n");
                                if (isopenretval != SYSM_OK)
                                {
                                        here->retval = isopenretval;
                                        close(localstruct->filedes);
                                        FREE(localstruct);
                                        here->monitordata = NULL;
                                        if (debug)
                                                printf("ending service_test_http() with retval = %s\n", errtostr(here->retval));
                                        return;
                                }
                                memset(buffer, 0, 1024);
				sprintf(buffer, "GET %s HTTP/1.0", here->checkent->url);
				sendline(localstruct->filedes, buffer);
				sendline(localstruct->filedes, "");
				
                                localstruct->state = WWW_SENT_REQUEST;
                                if (debug)
                                        printf("connected() to the http port of %s\n", here->checkent->hostname);
                        }
                        break;
                }
                case WWW_SENT_REQUEST:
                {
                        while (data_waiting_read(localstruct->filedes, 0))
                        {
				getline_tcp(localstruct->filedes, buffer);
                                if (debug)
                                        printf("http.c:Got :%s: Searching for :%s:\n", buffer, here->checkent->url_text);
				if (strlen(buffer) == 0)
				{
					localstruct->timesblank++;
					if (localstruct->timesblank > 10)
					{
						here->retval = SYSM_BAD_RESP;
					}
					break;
				}
				if (here->retval == SYSM_OK || 
					strlen(buffer) < strlen(here->checkent->url_text))
				{
					continue;
				}
				for (x = 0;x<(strlen(buffer)-strlen(here->checkent->url_text)); x++)
				{
					if (strncmp(buffer+x,here->checkent->url_text,strlen(here->checkent->url_text)) == 0)
					{
						if (debug) printf("http.c:Found it!\n");
		                                here->retval = SYSM_OK;
						break;
					}
		                }
                        }
                        break;
                }
                default:
                {
                        abort(); /* Muahaha */
                        break;
                }
        }
        if (here->retval != -1)
        {
                /* insert cleanup code here */
                close (localstruct->filedes);
                FREE(localstruct);
                here->monitordata = NULL;
        }
        return;
}

void
stop_test_www(struct monitorent *here)
{
        struct wwwdata *localstruct = NULL;
	
	localstruct = here->monitordata;
	if (localstruct == NULL)
		return;

	close(localstruct->filedes);
	FREE(localstruct);
	here->monitordata = NULL;
        return;
}

