/* $Id: icmp.c,v 1.19 1997/12/09 15:57:04 jared Exp $ */
#include "config.h"

extern int debug;
extern struct protoent *icmpproto;

#define ICMP_PACKET_SIZE 264
#define ICMP_HOLD_QUEUE (ICMP_PACKET_SIZE*200)

#define A(bit)          temp.rcvd_tbl[(bit)>>3]  /* identify byte in array */
#define B(bit)          (1 << ((bit) & 0x07))   /* identify bit in byte */
#define CLR(bit)        (A(bit) &= (~B(bit)))  /* do something... */

/* All the variables needed for pinging (I think) */
struct pingdata {
        struct sockaddr_in *to;
        struct ICMPHDR *icp;
        struct IPHDR *ip;
        int hlen;
        struct my_hostent *hp;
        int hold;
        u_char *datap, *packet;
        int counter;
        int icmp_s;                 /* Filedescriptor in use */
        int ident;                  /* process id to identify our packets */
        int packetsent;             /* number of packets sent */
	struct sockaddr whereto;        /* who to ping */
	u_char outpack[256];            /* the packet we output */
	char rcvd_tbl[128];		/* for doing bit shifts */
	int nreceived;                  /* # of packets we got back */
	struct timeval lastsentat;
} temp;

int in_cksum();
void pinger(struct pingdata*);

void	start_test_ping(struct monitorent *here)
{
	struct pingdata *localstruct;
	/* set things all up, and we should be Ok */

	here->monitordata = MALLOC(sizeof(struct pingdata));
	memset(here->monitordata, 0, sizeof(struct pingdata));
	localstruct = here->monitordata;

        gettimeofday(&here->lastserv, NULL);

	localstruct->nreceived = 0; /* zero it out */
	localstruct->packetsent = 0; /* zero it out */

	localstruct->datap = &localstruct->outpack[8 + sizeof(struct timeval)];

	/* initalize the variable */
	memset(&localstruct->whereto, 0, sizeof(struct sockaddr));

	if (debug) printf("setting up ping of host %s\n", 
		here->checkent->hostname);

	/* set to */
        localstruct->to=(struct sockaddr_in *)&localstruct->whereto;

	/* internet protocol */
        localstruct->to->sin_family = AF_INET;

        localstruct->to->sin_addr.s_addr = here->checkent->inaddr;

        /* do a dns lookup on the hostname we're being passed */
        localstruct->hp = my_gethostbyname(here->checkent->hostname);

        if (!localstruct->hp)  /* did we get an error doing a dns query */
	{
		/* if so, do the return thang */
		here->retval = SYSM_NODNS;
		FREE(localstruct);
		here->monitordata = NULL;
		return; /* can't forget this else we core */
	}

	/* set the family type */
	localstruct->to->sin_family=localstruct->hp->h_addrtype;

	memcpy((caddr_t)&localstruct->to->sin_addr, localstruct->hp->my_h_addr, 
		localstruct->hp->h_length);

        if (!(localstruct->packet = (u_char *)MALLOC(ICMP_PACKET_SIZE))) 
	{
                /* aie!! */
                (void)fprintf(stderr, "icmp.c: out of memory.\n");
                abort();
        }
        for (localstruct->counter = 8; localstruct->counter < 128;
		++localstruct->counter)  /* do something.. */
			*localstruct->datap++ = localstruct->counter;
					/* and keep doing it */

	/* generate our identity for the packet */
        localstruct->ident = (unsigned short int)rand() & 0xFFFF;

	if (debug)
		printf("Created ICMP identity id of %d\n", localstruct->ident);

        localstruct->icmp_s = socket(AF_INET, SOCK_RAW, icmpproto->p_proto);

        if (localstruct->icmp_s == -1) 
	{
                /* usually -EPERM */
                perror("ping: socket -- are you root? -- aborting!");
		/* I am always root! -msa */
		FREE(localstruct->packet); /* leak memory, never! */
                exit(1);
        }

        localstruct->hold = ICMP_HOLD_QUEUE;

	/* Set the size of our internal hold queue */
        (void)setsockopt(localstruct->icmp_s, SOL_SOCKET, 
		SO_RCVBUF, (char *)&localstruct->hold,
		sizeof(localstruct->hold));

	/*  Send a ping */
	pinger(localstruct);

	if (debug)
		printf("Sent an ICMP echo-request to %s\n",
			here->checkent->hostname);

	/* Track the last time that we sent a ping */
	gettimeofday(&localstruct->lastsentat, NULL);
	
	/* end of setup, we send the packet, and leave it at that  -- 
		service_test_ping watches for replies*/

	return;
}

void	service_test_ping(struct monitorent *here)
{
	struct pingdata *localstruct = NULL;
	struct timeval right_now;
	struct sockaddr_in from;
	int cc = 0, fromlen = 0;

	/* This must be done first, else we're pointing to nowhere land */
	localstruct = here->monitordata;
	
	if (localstruct == NULL)
	{
		printf("bug - localstruct == NULL in icmp.c:service_test_ping\n");
		return;
	}
        gettimeofday(&here->lastserv, NULL);

	/* Do some time calculation: */

	gettimeofday(&right_now, NULL);

	if (mydifftime(localstruct->lastsentat, right_now) <= 1)
		return;
		/* It's not time yet to watch for more packets */

	/* watch for imcp echo-replies and send more
		echo requests if we need to */

	/* If we've sent too many icmp echo-requests, bail out */
	if (debug)
	{
		fprintf(stderr, "icmp.c:service_test_ping:While pinging %s, packetsent = %d\n",
			here->checkent->hostname, localstruct->packetsent);
	}

	if (localstruct->packetsent > MAX_TRIES)
	{
                /* if so, do the return thang */
                here->retval = SYSM_UNPINGABLE; /* It's not pingable */
		close(localstruct->icmp_s); /* damnit */
		FREE(localstruct->packet); /* Free our packet */
		FREE(localstruct); /* Free memory we'd normally leak */
		here->monitordata = NULL; /* tag memory as freed internally */
		return; /* go back to where we came from */
	}

	while (data_waiting_read(localstruct->icmp_s, 0) == 1)
	{ 
		/* we've got some data */
		if (debug) fprintf(stdout, "icmp.c:service_test_ping:select says theres data\n");

	        fromlen = sizeof(from); /* no comment */

	        if ((cc = recvfrom(localstruct->icmp_s, 
			(char *)localstruct->packet, ICMP_PACKET_SIZE, 0,
			(struct sockaddr *)&from, &fromlen)) < 0)
		{
			if (errno == EINTR) /* if we get interrupted */
			{
				/* Decrease the counter */
				localstruct->counter--;
				continue; /* and try again */
			}
			perror("ping: recvfrom");
			localstruct->counter--; /* decrease the counter */
			continue; /* try again */
		}
		/* determine if it was ours */

		/* Check the IP header */
		localstruct->ip = (struct IPHDR *)localstruct->packet;
		localstruct->hlen = localstruct->ip->IHL << 2;

		/* Now the ICMP part */
		localstruct->icp=(struct ICMPHDR *)(localstruct->packet + 
			localstruct->hlen);

		/* Check to see if it's an ICMP_ECHOREPLY */
		if (localstruct->icp->ICMP_TYPE == ICMP_ECHOREPLY)
		{
			/* Do some debugging and print out what we
			   generated with what we got back */
#ifdef HIGHDEBUG
			if (debug)
			{
				printf("Comparing ICMP id %d to %d\n", 
					localstruct->icp->ICMP_ECHO_ID, 
					localstruct->ident);
			}
#endif /* HIGHDEBUG */

			/* Determine if it was ours */
			if (localstruct->icp->ICMP_ECHO_ID==localstruct->ident)
			{
					/* Increment our count */
		                        localstruct->nreceived++;
			}
		}
	} /* end while */

	gettimeofday(&right_now, NULL);
        if ((localstruct->nreceived == 0) &&
                (mydifftime(localstruct->lastsentat, right_now) >= 1))
        {
		/* Send another ping */
                pinger(localstruct);
		if (debug)
		        printf("Sent an ICMP echo-request to %s\n",
				here->checkent->hostname);
	
		/* Track it */
                gettimeofday(&localstruct->lastsentat, NULL);
                return;
        }

	/* Execution should reach here only upon one circumstance:
		We received an ICMP response from the host in question.
	 */

	/* Some basic housekeeping */
        if (close(localstruct->icmp_s) == -1)
        {
                /* sigh, can't close.. something isn't right here */
                perror("icmp.c: close");
        }

	here->retval = SYSM_OK;
	if (debug)
		printf("Got an ICMP reply from %s\n", here->checkent->hostname);

	/* damnit leaking aren't we */
	FREE(localstruct->packet);

	/* Free memory we'd normally leak */
	FREE(localstruct);
	
	here->monitordata = NULL;

	return;
}

/*
 * pinger --
 *      Compose and transmit an ICMP ECHO REQUEST packet.  The IP packet
 * will be added on by the kernel.  The ID field is a random ID
 * and the sequence number is an ascending integer.  The first 8 bytes
 * of the data portion are used to hold a UNIX "timeval" struct in VAX
 * byte-order, to compute the round-trip time.
 *
 * 
 * bug --
 *	Our sense of time will slowly skew (i.e., packets will not be
 * launched exactly at 1-second intervals).
 *
 * This is not important in the network monitoring world...
 */
void	pinger(struct pingdata *localdata)
{
	int cc, ret1, waittime;

	localdata->packetsent++;
	localdata->icp = (struct ICMPHDR *)localdata->outpack;/* cast it */
	localdata->icp->ICMP_TYPE = ICMP_ECHO;	/* It's an ICMP_ECHO */
	localdata->icp->ICMP_CODE = 0;		/* ? */
		/* zero out the checksum */
	localdata->icp->ICMP_CHECKSUM = 0;
		/* put the sequence in the packet */
	localdata->icp->ICMP_SEQ = localdata->packetsent;
		/* stick the icmp id in the packet */
	localdata->icp->ICMP_ECHO_ID = localdata->ident;

	CLR(localdata->icp->ICMP_SEQ % 1024);	/* Clear it */

	cc = 128 + 8;			/* skips ICMP portion */

	/* compute ICMP checksum here */
	localdata->icp->ICMP_CHECKSUM = in_cksum((u_short *)localdata->icp, cc);

	/* send the packet */
	ret1 = sendto(localdata->icmp_s, (char *)localdata->outpack, 
		cc, 0, &localdata->whereto, sizeof(struct sockaddr));

	if (ret1 < 0 || ret1 != cc)  {
		perror("icmp.c:sendto");
		/* Unable to ping the site -- due to routing, usw */
	}

	/* end of sending packet */

	/* if ... then .. */
	if (!MAX_TRIES || localdata->packetsent < MAX_TRIES)
	{
		/* do nothing */
	} else {
		if (localdata->nreceived) 
		{
			waittime = 2 / 1000;
			if (!waittime)
				waittime = 1;
		} else
			waittime = MAX_TRIES;
	}
}

/*
 * in_cksum --
 *	Checksum routine for Internet Protocol family headers (C Version)
 */
int	in_cksum(u_short *addr, int len)
{
	int sum = 0, nleft = len;
	u_short answer = 0, *w = addr;

	/*
	 * Our algorithm is simple, using a 32 bit accumulator (sum), we add
	 * sequential 16 bit words to it, and at the end, fold back all the
	 * carry bits from the top 16 bits into the lower 16 bits.
	 */
	while (nleft > 1) 
	{
		sum += *w++;
		nleft -= 2;
	}

	/* mop up an odd byte, if necessary */
	if (nleft == 1) 
	{
		*(u_char *)(&answer) = *(u_char *)w ;
		sum += answer;
	}

	/* add back carry outs from top 16 bits to low 16 bits */
	sum = (sum >> 16) + (sum & 0xffff);	/* add hi 16 to low 16 */
	sum += (sum >> 16);			/* add carry */
	answer = ~sum;				/* truncate to 16 bits */

	return(answer);
}

void    stop_test_ping(struct monitorent *here)
{

        struct pingdata *localstruct = NULL;

        localstruct = here->monitordata;

        close(localstruct->icmp_s);
        FREE(localstruct);
        here->monitordata = NULL;
        return;

}
