/* $Id: loadconfig.c,v 1.49 1998/01/14 11:20:20 jared Exp $ */

#include "config.h"

extern int numfailures;
extern struct hostinfo *first;
extern int facility;
extern int debug;
extern char *statusfilename;
extern int html;
extern char *errorsto;
extern int pageinterval;
extern int showupalso;
extern int heartbeat;
extern int dnslog;
extern int dnsexpire;
extern int gotsighup;

int match_facility(char *factomatch)
{
	if (strcmp(factomatch, "kern") == 0) 
		return LOG_KERN;
	if (strcmp(factomatch, "user") == 0) 
		return LOG_USER;
	if (strcmp(factomatch, "mail") == 0) 
		return LOG_MAIL;
	if (strcmp(factomatch, "daemon") == 0) 
		return LOG_DAEMON;
	if (strcmp(factomatch, "auth") == 0) 
		return LOG_AUTH;
	if (strcmp(factomatch, "syslog") == 0) 
		return LOG_SYSLOG;
	if (strcmp(factomatch, "lpr") == 0) 
		return LOG_LPR;
	if (strcmp(factomatch, "news") == 0) 
		return LOG_NEWS;
	if (strcmp(factomatch, "uucp") == 0) 
		return LOG_UUCP;
	if (strcmp(factomatch, "cron") == 0) 
		return LOG_CRON;
#ifdef LOG_AUTHPRIV
	if (strcmp(factomatch, "authpriv") == 0) 
		return LOG_AUTHPRIV;
#endif /* LOG_AUTHPRIV */
	if (strcmp(factomatch, "local0") == 0) 
		return LOG_LOCAL0;
	if (strcmp(factomatch, "local1") == 0) 
		return LOG_LOCAL1;
	if (strcmp(factomatch, "local2") == 0) 
		return LOG_LOCAL2;
	if (strcmp(factomatch, "local3") == 0) 
		return LOG_LOCAL3;
	if (strcmp(factomatch, "local4") == 0) 
		return LOG_LOCAL4;
	if (strcmp(factomatch, "local5") == 0) 
		return LOG_LOCAL5;
        if (strcmp(factomatch, "local6") == 0) 
		return LOG_LOCAL6;
        if (strcmp(factomatch, "local7") == 0) 
		return LOG_LOCAL7;
	if (strcmp(factomatch, "none") == 0) /* user requsted no syslogging */
		return -2;

	return -1; /* no facility matches */
}


struct hostinfo *loadconfig(char *cfname)
{
	struct hostinfo *head = NULL;
	FILE *cnf; /* file handle to take care of the config file loading */

	/* reset it */
	int ln = 0;
	int lvl = 0;

	cnf = fopen(cfname, "r"); /* open the file */

	if (cnf == NULL)
	{
		perror(cfname);
		return NULL;
	}

	fprintf(stdout, "Loading config file..\n"); /* log what's going on */

	head = parseline(&ln, &lvl, cfname, cnf); /* actually load it */
	if (lvl != 0)
	{
		syslogmsg("Parse error in config file, misbalanced braces\n");
		fprintf(stderr, "Parse error in config file, misbalanced braces\n");
		exit(1);
	}

	if (head == NULL) /* check for weird errors */
		syslogmsg("loading of config file returned NULL (empty cnfg?)");

	fclose(cnf); /* leaking a fd here aren't we */

	fprintf(stdout, "Done loading config file..\n");

	syslogmsg("done loading config file");

	return head;
}

/* Free the n-ary linked list */
void	free_it(struct hostinfo *doit)
{

	if (doit == NULL)
		return;

	if (doit->child != NULL)
		free_it(doit->child);

	if (doit->sibling != NULL)
		free_it(doit->sibling);

	if (doit->sibling != NULL)
		if ((doit->sibling->sibling == NULL) &&
			(doit->sibling->child == NULL))
		{
			FREE(doit->sibling);	
			doit->sibling = NULL;
		}

	if (doit->child != NULL)
		if (doit->child->sibling == NULL && doit->child->child == NULL)
		{
			FREE(doit->child);	
			doit->child = NULL;
		}
	return;
}

void free_tree(struct hostinfo *freemebaby)
{
	if (freemebaby == NULL)
		return;
	free_it(freemebaby);
	if (freemebaby->sibling == NULL && freemebaby->child == NULL)
		FREE(freemebaby);
	else 
		syslogmsg("bug in freeing memory\n");
	
	/* free the binary tree freemebaby */
}

void reload_config()
{
	/* log a message */
	syslogmsg("caught SIGHUP preparing to reload config");
	signal(SIGHUP, reload_config);
	
	gotsighup = 1; /* we need to watch for this, and when 
		it gets set, bail out of checks and do some cleanup, then
		copy alerts over, then free old tree */

	return;
}

void find_match(struct hostinfo *matchme,struct hostinfo *righthere)
{
	if (righthere == NULL)
		return;

	if ((strcmp(matchme->hostname, righthere->hostname) == 0) &&
		(matchme->type == righthere->type) && 
		(matchme->port == righthere->port))
	{

		hard_copy(matchme, righthere);
		return;
	}

	find_match(matchme, righthere->child); /* check the kids */
	find_match(matchme, righthere->sibling); /* check the siblings */
}

void	hard_copy(struct hostinfo *old, struct hostinfo *new)
{
	struct hostinfo *tmp1, *tmp2;

	tmp1 = new->child;
	tmp2 = new->sibling;

	if (debug)
	printf("copying %s %d %d over %s %d %d\n", old->hostname, old->port,
		old->lastcheck, new->hostname, new->port, new->lastcheck);
	
	new->lastcheck = old->lastcheck;
	new->downct = old->downct;
	new->max_down = old->max_down;
	new->contacted = old->contacted;
	new->lchecktime = old->lchecktime;
	new->deathtime = old->deathtime;
	new->last_up = old->last_up;
}

void copy_alerts(struct hostinfo *old, struct hostinfo *new)
{
	if (old == NULL)
		return;

	find_match(old, new);

	copy_alerts(old->child, new);
	copy_alerts(old->sibling, new);
}

struct hostinfo *sync_after_sighup(struct hostinfo *oldhead, char *cnffile)
{
	struct hostinfo *newhead = NULL;
	
	if (debug)
		syslogmsg("entering sync_after_sighup");

	/* load the config file */
	newhead = loadconfig(cnffile);

	if (debug)
		syslogmsg("calling copy_alerts");

	/* copy the alerts */
	copy_alerts(oldhead, newhead);

	if (debug)
		syslogmsg("exiting sync_after_sighup");

	/* free the old tree */
	free_tree(oldhead);
	oldhead = NULL;

	if (newhead == oldhead)
		syslogmsg("BUG! newhead == oldhead ");

	return newhead;
}

struct hostinfo	*parseline(int *ln, int *lvl, char *cfname, FILE *fh)
{
        char buf[256];
#ifdef OLDWAY
	char a[100], b[100], c[100], d[100], e[100], f[100], g[100];
#endif /* OLDWAY */
        struct my_hostent *hp;
	struct parsed mydata;

        struct hostinfo data, *xtra = NULL, *myfst = NULL, *current = NULL;

	while (fgets(buf, 250, fh) != NULL)
{
        *ln = *ln+1;

#ifdef REIMPLEMENT_SOMEHOW_ELSE
        for (vars = 0; vars < strlen(buf); vars++)
                buf[vars] = tolower(buf[vars]);
#endif /* REIMPLEMENT_SOMEHOW_ELSE */

	memset(&data, 0, sizeof(struct hostinfo)); /* init it the ugly way */

        data.child = NULL; /* nothing linked in yet... */
        data.sibling = NULL; /* My next brother/sister */
        data.contact[0] = '\0'; /* init it */
        data.message[0] = '\0'; /* init it */
        data.port = 0; /* init it */
        data.lastcheck = 0; /* init it */
	data.contacted = FALSE; /* init it */
	data.queued = 0; /* initalize it */
	data.downct = 0; /* init it */
	data.max_down =  numfailures; /* init it */
	data.lchecktime = 0;
	data.last_up = time(NULL);
	data.deathtime = time(NULL);

#ifdef OLDWAY
        vars = 0;

        vars = sscanf(buf, "%s %s %s %s %s %s %s",a,b,c,d,e,f,g);
#endif /* OLDWAY */
	if (parse(buf, &mydata) == -1)
	{
		printf("parseline: Invalid line %s:%d\n", cfname, *ln);
		continue;
	}

	if (debug)
		printf ("line #%d - strlen() = %d\n", *ln, strlen(buf));

	if (strlen(buf) == 1)       
		continue; /* if a blank line */

        if (mydata.count == 0)              
		continue; /* if a blank line */
        if ((*mydata.data[0]==';') || (*mydata.data[0]=='#')) 
		continue; /* comment */
        if (*mydata.data[0] == '}')
        {
                *lvl = *lvl -1;
		/* Free the structure */
		free_parsed(&mydata);

                return myfst;
        } else {
                if (mydata.count < 3)
                {
                        fprintf(stderr, "%s:%d: Not enough args\n",cfname, *ln);
                        continue;
                }
	}
	if (strncmp(mydata.data[0], "config", 6) == 0)
	{
		if (strncmp(mydata.data[1], "include", 7) == 0)
		{
			int subln;
			FILE *subfh;
			struct hostinfo *subf;
			if (mydata.count != 3)
				continue;

			printf("about to open sub file %s\n", mydata.data[2]);
			subfh = fopen(mydata.data[2], "r");
			if (subfh == NULL)
			{
				perror(mydata.data[2]);
				continue;
			}
			/* parse the subfile */
			subf=parseline(&subln, lvl, mydata.data[2], subfh);
			/* subf now equal to the tree generated from
			 * that other file */

			xtra = subf;
			/* close the fd */
			fclose(subfh);
			goto setup_data;
			
		}
		if (strncmp(mydata.data[1], "showupalso", 10) == 0)
		{
			/* Show hosts that are up also in text file */
			showupalso = 1;
			continue;
		} else
		if (strncmp(mydata.data[1], "errorsto", 8) == 0)
		{
			if (debug)
				printf("Setting errorsto == %s\n", mydata.data[2]);
			
			errorsto = MALLOC(strlen(mydata.data[2])+1);
			strcpy(errorsto,mydata.data[2]);
			continue;
		} else
		if (strncmp(mydata.data[1], "noheartbeat", 11) == 0)
		{
			heartbeat = 0;
			continue;
		} else
		if (strncmp(mydata.data[1], "pageinterval", 12) == 0)
		{
			pageinterval = atoi(mydata.data[2]); /* in minutes */
			continue;
		} else
		if (strncmp(mydata.data[1], "logging", 7) == 0)
		{
			facility = match_facility(mydata.data[2]);
			if (facility == -1)
			{
				fprintf(stderr, "logging facility of %s invalid, using daemon instead\n", mydata.data[2]);
				facility = LOG_DAEMON;
			}
			continue;
		} else
		if (strncmp(mydata.data[1], "dnslog", 6) == 0)
		{
			dnslog = atoi(mydata.data[2]);
			continue;
		} else
		if (strncmp(mydata.data[1], "dnsexpire", 9) == 0)
		{
			dnsexpire = atoi(mydata.data[2]);
			continue;
		} else
		if (strncmp(mydata.data[1], "statusfile", 10) == 0)
		{
			if (mydata.count != 4)
			{
				fprintf(stderr, "invalid number of args for configuration option %s\n", mydata.data[1]);
				continue;
			}
			if (strncmp(mydata.data[2], "html", 4) == 0) html = 1;
			if (strncmp(mydata.data[2], "text", 4) == 0) html = 0;
			if (html == -1)
				fprintf(stderr, "statusfile type of %s is invalid, try either html or text\n", mydata.data[2]);
			statusfilename = MALLOC(strlen(mydata.data[3])+1);
			strcpy(statusfilename, mydata.data[3]);

			continue;
		} else if (strncmp(mydata.data[1], "numfailures", 11) == 0)
		{
			numfailures = atoi(mydata.data[3]);
	
			if (debug)
				fprintf(stderr, "changing numfailures to %d\n",
					numfailures);

			continue;
		} else if (strncmp(mydata.data[1], "sleeptime", 9) == 0)
		{
			fprintf(stderr, "Ignoring legacy configuration option\n");
		} else {
			fprintf(stderr, "Unknown configuration option %s\n",
				mydata.data[1]);
		}
	} /* end if config block */
	else if (strncmp(mydata.data[1], "ping",4) == 0)
	{
                data.type = SYSM_TYPE_PING; /* ping */
	}
	else if (strncmp(mydata.data[1], "pop3", 4) == 0)
	{
		data.type = SYSM_TYPE_POP3; /* pop3 */
	}
	else if (strncmp(mydata.data[1], "imap", 4) == 0)
	{
		data.type = SYSM_TYPE_IMAP; /* imap */
		data.port = IMAP_PORTNUM;
	}
        else if (strncmp(mydata.data[1], "tcp",3) == 0)
	{
                data.type = SYSM_TYPE_TCP; /* tcp */
	}
        else if (strncmp(mydata.data[1], "udp",3) == 0)
	{
                data.type = SYSM_TYPE_UDP; /* udp */
	}
	else if (strncmp(mydata.data[1], "nntp", 4) == 0)
	{
		data.type = SYSM_TYPE_NNTP; /* NNTP - see nntp.c for notes */
		data.port = NNTP_PORTNUM;
	}
	else if (strncmp(mydata.data[1], "smtp", 4) == 0)
	{
		data.type = SYSM_TYPE_SMTP; /* SMTP */
		data.port = SMTP_PORTNUM; /* set the portnumber */
	}
	else if (strncmp(mydata.data[1], "umichx500", 9) == 0)
	{
		data.type = SYSM_TYPE_X500; /* see umichX500.c for notes */
		data.port = 1111; /* umich local hack */
	}
	else if (strncmp(mydata.data[1], "www", 3) == 0)
	{
		if (debug)
			printf("type = www content\n");
		data.type = SYSM_TYPE_WWW; /* web content */
		data.port = HTTP_PORTNUM; /* set the portnumber */
	}
#ifdef IMPLEMENTED
	else if (strncmp(mydata.data[1], "authdns", 7) == 0)
	{
		data.type = SYSM_TYPE_DNS;
		data.port = DNS_PORTNUM;
	}
	else if (strncmp(mydata.data[1], "radius", 6) == 0)
	{
		data.type = SYSM_TYPE_RADIUS;
		data.port = RADIUS_PORTNUM;
	}
	else if (strncmp(mydata.data[1], "https", 5) == 0)
	{
		data.type=SYSM_TYPE_HTTPS;
		data.port=HTTPS_PORTNUM;
	}
#endif /* IMPLEMENTED */
	else 
	{
                fprintf(stderr, "line %d: Invalid test type\n", *ln);
                continue;
        }
        strcpy(data.hostname, mydata.data[0]); /* note hostname */
#ifdef IMPLEMENTED
	if (data.type == SYSM_TYPE_DNS) 
	{
		if (mydata.count < 4) {
			fprintf(stderr,"line %d: Bad args to authdns\n",*ln);
			continue;
		}
		strcpy(data.username, mydata.data[2]);
		if (mydata.count == 4) strcpy(data.contact, mydata.data[3]);
	}
	else if (data.type == SYSM_TYPE_RADIUS) 
	{
		if (mydata.count < 6) {
			fprintf(stderr,"line %d: Bad args to radius\n", *ln);
			continue;
		}
		strcpy(data.username, mydata.data[2]);
		strcpy(data.password, mydata.data[3]);	
		strcpy(data.url, mydata.data[4]);
		if (mydata.count == 6) strcpy(data.contact, mydata.data[5]);
	}
#endif /* IMPLEMENTED */
	if (data.type == SYSM_TYPE_POP3) /* pop3 */
	{
		if (debug)
			printf("type == pop3, args = %d\n", mydata.count);
		if (mydata.count < 5) /* check for bad # of args */
		{
			fprintf(stderr,
				"line %d: Invalid number of args to pop3\n", *ln);
			continue;
		}
		strcpy(data.username, mydata.data[2]);
		strcpy(data.password, mydata.data[3]);
		if (debug)
			printf("username is %s password is %s\n", 
				data.username, data.password);
		strcpy(data.message, mydata.data[4]); /* message */
		if (debug)
			printf("Message to send is %s\n", data.message);
		if (mydata.count == 6)
		{
			strcpy(data.contact, mydata.data[5]);
			if (debug)
				printf("contact is %s\n", data.contact);
		}
		
	}  else if (data.type == SYSM_TYPE_IMAP) /* imap */
	{
		if (debug)
			printf("type == imap, args = %d\n", mydata.count);
		if (mydata.count < 5) /* check for bad # of args */
		{
			fprintf(stderr, "line %d: Invalid number of args\n"
				, *ln);
			continue;
		}
		strcpy(data.username, mydata.data[2]);
		strcpy(data.password, mydata.data[3]);
		if (debug)
			printf("username is %s password is %s\n", 
				data.username, data.password);
		strcpy(data.message, mydata.data[4]); /* message */
		if (debug)
			printf("Message to send is %s\n", data.message);
		if (mydata.count == 6)
		{
			strcpy(data.contact, mydata.data[5]);
			if (debug)
				printf("contact is %s\n", data.contact);
		}
		
	} else
        if (data.type == SYSM_TYPE_TCP || data.type == SYSM_TYPE_UDP)
        {
                if (mydata.count < 4) /* check for bad # of args */
                {
                        fprintf(stderr,"line %d: Invalid number of args\n",*ln);
                        continue;
                }
                data.port = atoi(mydata.data[2]);
                if (0 == data.port) /* check for bad port #'s */
                {
                        fprintf(stderr, "line %d: Invalid port number\n", *ln);
                        continue;
                }
                strcpy(data.message, mydata.data[3]); /* message */
                if (mydata.count == 5) /* if there is a contact */
                        strcpy(data.contact, mydata.data[4]);
	} else if ( (data.type == SYSM_TYPE_WWW) || 
		(data.type==SYSM_TYPE_HTTPS))  /* www content */
	{
		if (debug)
			printf("parsing  %s\n", buf);
		if (mydata.count < 5)
		{
			printf("insufficent args for www test on line %d\n",*ln);
			continue;
		}
		strcpy(data.url, mydata.data[2]);
		strcpy(data.url_text, mydata.data[3]);
		strcpy(data.message, mydata.data[4]);
		if (debug)
		{
			printf("set url to be %s\n", data.url);
			printf("set url_text to be %s\n", data.url_text);
			printf("set message to be %s\n", data.message);
		}
		if (mydata.count > 5)
		{
			strcpy(data.contact, mydata.data[5]);
			if (debug)
			printf("set contact to be %s\n", data.contact);
		}
	        /* data now contains all tcp/udp stuff that will be required */
        } else {
                /* ping */
                strcpy(data.message, mydata.data[2]); /* message */
                if ((mydata.count > 3) && (mydata.data[3][0] != '{'))
                {
                        strcpy(data.contact, mydata.data[3]);
                        if ((mydata.count == 5) && (mydata.data[4][0] == '{'))
                        {
                                /* there are children */
                                *lvl = *lvl+1;
                                data.child =  parseline(ln, lvl, cfname, fh);
                        }
                        else if (mydata.count >= 5)
			{
				fprintf(stderr, "line %d: Parse error on ping service line\n", *ln);
			}
                } else if (mydata.count == 4 && mydata.data[3][0] == '{') {
                        /* there are children */
                        *lvl = *lvl+1;
                        data.child = parseline(ln, lvl, cfname, fh);
                }
        }

        /* Ok, time to allocate some memory */
        hp = my_gethostbyname(data.hostname); /* do dns query */
        if (!hp) 
        {
                fprintf(stderr, "%s: unknown host\n", data.hostname);
		fprintf(stderr, "Ignoring line\n");
		data.lastcheck = SYSM_NODNS;
		continue;
        } else 
		data.inaddr = inet_addr(data.hostname);   /* do conversion */

        xtra = MALLOC(sizeof(struct hostinfo));

        /* copy the data structure into the xtra pointer we allocated */
        memcpy(xtra, &data, sizeof(struct hostinfo));

setup_data:
	/* if the first entry is null */
        if (myfst == NULL)
        {
                current = xtra;
                myfst = xtra; /* my first entry is now equal to the
                                one that we just made -- same w/ current */
        } 
	else 
	{
                current->sibling = xtra;
                current = xtra;
        }

	} /* end of while */
	return myfst;
}
