/* $Id: nntp.c,v 1.11 1997/11/23 15:10:28 jared Exp $ */
#include "config.h"

extern int debug;

/* Does a rfc nntp check to see if the server is responding -- 

RFC's important to News:
        RFC 977: NNTP (Phil Lapsley and Brian Kantor)
        RFC 1036: Standard for Interchange of USENET Messages
                (M.Horton, Rick Adams)

	Excerpt from rfc 977:
2.4.2.  Status Responses

   These are status reports from the server and indicate the response to
   the last command received from the client.

   Status response lines begin with a 3 digit numeric code which is
   sufficient to distinguish all responses.  Some of these may herald
   the subsequent transmission of text.

   The first digit of the response broadly indicates the success,
   failure, or progress of the previous command.

      1xx - Informative message
      2xx - Command ok
      3xx - Command ok so far, send the rest of it.
      4xx - Command was correct, but couldn't be performed for
            some reason.
      5xx - Command unimplemented, or incorrect, or a serious
            program error occurred.

 */

/* returns status based upon this:

  0 = ok,  1 = conn ref, 2 = ENETUNREACH, 3 = EHOSTDOWN||EHOSTUNREACH,
  4 = ETIMEDOUT, 5 = no dns entry, 6 = unpingable, 
  7 = Throttled, 8 = no auth, 9 = no response

 */

struct nntpdata {
        int filedes;
        time_t start; /* test start time */
        time_t lastact; /* last activity */
        int state;
        };

#define NNTP_CONNECT    1 /* doing a connect() */
#define NNTP_WAITBAN    2 /* Waiting for the banner */
#define NNTP_SENT_QUIT  3 /* sent a QUIT */
#define NNTP_GOT_BYE    4 /* got a bye from the remote server */

void	start_test_nntp(struct monitorent *here)
{
	/* start the socket connecting to the remote host, and then
	   let service_test_nntp take care of the rest
	 */

        struct nntpdata *localstruct = NULL;
        struct my_hostent *hp = NULL;
        struct sockaddr_in name;
        int errcode = 0;

        /* Allocate our memory */
        here->monitordata = MALLOC(sizeof(struct nntpdata));

        localstruct = here->monitordata;

        time(&localstruct->start);

        localstruct->lastact = localstruct->start;

        localstruct->filedes = open_sock();

        if (localstruct->filedes == -1)
		abort();	/* Bad error, code around it later */

        hp = my_gethostbyname(here->checkent->hostname);

        /* zero out the space */
        memset ( &name, 0, sizeof ( name ) );

        /* copy data */
        memcpy((char*)&name.sin_addr, (char*)hp->my_h_addr, hp->h_length);

        /* set family type */
        name.sin_family = AF_INET;

        /* set the port we're connecting to */
        name.sin_port = htons(here->checkent->port);

        if (debug)
                printf("start_test_nntp() doing connect() to %s:%d\n",
                        here->checkent->hostname, here->checkent->port);

        errcode = connect(localstruct->filedes, (struct sockaddr*)&name,
                sizeof(struct sockaddr_in));

        if ((errcode < 0) && (errno != EINPROGRESS))
        {
                if (close(localstruct->filedes) == -1)
                        abort(); /* close the fd incase it's left */

		switch(errno)
		{
			case ECONNREFUSED: 
			case EINTR:
				here->retval = SYSM_CONNREF;
				break;
			case ENETUNREACH:
				here->retval = SYSM_NETUNRCH;
				break;
			case EHOSTDOWN:
			case EHOSTUNREACH:
				here->retval = SYSM_HOSTDOWN;
				break;
			case ETIMEDOUT:
				here->retval = SYSM_TIMEDOUT;
				break;
			default:
				break;
		}

                /* Free memory we'd normally leak */
                FREE(localstruct);
                here->monitordata = NULL;

                return;
        }

        /* doing a connect() */
        localstruct->state = NNTP_CONNECT;

        /* poll it again later */
        return;
}

void	service_test_nntp(struct monitorent *here)
{
        /* wait for the fd to be readable/writable, then read the banner,
           and send a QUIT if appropriate so that we can return the error
           code for the tcp connecton problem, or for the nntp response.
         */
        /* do the actual real parts of the checks after
           it's been set up */

        struct nntpdata *localstruct = NULL;
        char buffer[256];
        int isopenretval = -1;

        /* do some variable shufflign */
        localstruct = here->monitordata;
	if (localstruct == NULL)
	{
		printf("bug! localstruct == NULL in nntp.c:service_test_nntp\n");
		return;
	}

	if (debug)
		printf("service_test_nntp: state = %d\n", localstruct->state);
        /* do different things based on the state */
        switch (localstruct->state)
        {
                case NNTP_CONNECT:
                {
                        isopenretval = is_open(localstruct->filedes);
			if (debug)
   			    printf("is_open is returning %d\n",isopenretval);
                        if ((isopenretval != -1) &&
                                (isopenretval != SYSM_INPROG))
                        {
                                time(&localstruct->lastact);
                                if (debug)
                                        printf("is_open() in service_test_nntp()\n");
                                if (isopenretval != SYSM_OK)
                                {
                                        here->retval = isopenretval;
                                        close(localstruct->filedes);
                                        FREE(localstruct);
                                        here->monitordata = NULL;
                                        if (debug)
                                                printf("ending service_test_nntp() with retval = %s\n", errtostr(here->retval));
                                        return;
                                }
                                localstruct->state = NNTP_WAITBAN;
                                if (debug)
                                        printf("connected() to the nntp port of %s\n", here->checkent->hostname);
                        }
                        break;
                }
                case NNTP_WAITBAN:
                {
                        if (data_waiting_read(localstruct->filedes, 0))
                        {
                                memset(buffer, 0, 256);
                                read(localstruct->filedes, buffer, 254);
                                buffer[strlen(buffer)-1] = '\0';
                                if (debug)
                                        printf("Got :%s:\n", buffer);
                                if (strncmp(buffer, "200 ", 4) == 0)
                                {
                                        localstruct->state = NNTP_SENT_QUIT;
                                } else {
                                        here->retval = SYSM_BAD_RESP;
                                }
                                sendline(localstruct->filedes, "QUIT");
                        }
                        break;
                }
                case NNTP_SENT_QUIT:
                {
                        if (data_waiting_read(localstruct->filedes, 0))
                        {
                                memset(buffer, 0, 256);
                                read(localstruct->filedes, buffer, 254);
                                buffer[strlen(buffer)-1] = '\0';
                                if (debug)
                                        printf("Got2:%s:\n", buffer);
                                if (strncmp(buffer, "205 ", 4) == 0)
                                        localstruct->state = NNTP_GOT_BYE;
                                else
                                        here->retval = SYSM_BAD_RESP;
                        }
                        break;
                }
                case NNTP_GOT_BYE:
                {
                        here->retval = SYSM_OK;
                        if (debug)
                                printf("nntp: returing a OK value\n");
                        break;
                }
                default:
                {
                        abort(); /* Muahaha */
                        break;
                }
        }
        if (here->retval != -1)
        {
                /* insert cleanup code here */
                close(localstruct->filedes);
                FREE(localstruct);
                here->monitordata = NULL;
        }
        return;

}

void
stop_test_nntp(struct monitorent *here)
{
        struct nntpdata *localstruct = NULL;
	
	localstruct = here->monitordata;

	if (localstruct == NULL)
		return;

	close(localstruct->filedes);
	FREE(localstruct);
	here->monitordata = NULL;
        return;
}

