/* $Id: pop3.c,v 1.9 1997/11/23 15:10:28 jared Exp $ */
#include "config.h"

extern int debug;

struct pop3data {
	int filedes;
	time_t start;
	time_t lastact;
	int state;
	};

#define POP3_CONNECT	1 /* Doing the connect() */
#define POP3_WAITBAN	2 /* Waiting for the banner of +OK */
#define POP3_SENT_USER 	3 /* sent our username */
#define POP3_SENT_PASS  4 /* sent our password */
#define POP3_SENT_QUIT	5 /* sent a QUIT */
#define POP3_GOT_BYE	6 /* got a +OK saying goodbye */

/* Notes :

	Some pop servers disconnect after a bad password automagically,
	so we need to trap that properly if we can't send a quit, don't
	give any protocol mismatch errors 
 */

void	start_test_pop3(struct monitorent *here)
{

        /* set up the check, start to connect() to the remote host
           then let service_test_pop3() watch for the fd to be
           writable */

        struct pop3data *localstruct = NULL;
        struct my_hostent *hp = NULL;
        struct sockaddr_in name;
        int errcode = 0;

        /* Allocate our memory */
        here->monitordata = MALLOC(sizeof(struct pop3data));

        localstruct = here->monitordata;

        time(&localstruct->start);

	localstruct->state = POP3_CONNECT;

        localstruct->filedes = open_sock();

        if (localstruct->filedes == -1)
        {
                /* Bad error, code around it later */
                abort();
        }

        hp = my_gethostbyname(here->checkent->hostname);

        /* zero out the space */
        memset ( &name, 0, sizeof ( name ) );

        /* copy data */
        memcpy((char*)&name.sin_addr, (char*)hp->my_h_addr, hp->h_length);

        /* set family type */
        name.sin_family = AF_INET;

        /* set the port we're connecting to */
        name.sin_port = htons(POP3_PORTNUM);

	if (debug)
	        printf("start_test_pop3() doing connect() to %s:%d\n",
	                here->checkent->hostname, POP3_PORTNUM);

        /* do the connect() here: */
        errcode = connect(localstruct->filedes, (struct sockaddr*)&name,
                sizeof(struct sockaddr_in));

	/* poll it again later */
	return;
}

void	service_test_pop3(struct monitorent *here)
{
        /* do the actual real parts of the checks after
           it's been set up */

        struct pop3data *localstruct = NULL;
        char buffer[256];
        int isopenretval = -1;

        /* do some variable shufflign */
        localstruct = here->monitordata;
	if (localstruct == NULL)
	{
		printf("bug! localstruct == NULL in pop3.c:service_test_pop3\n");
		return;
	}

        if (debug)
                printf("service_test_pop3: state = %d\n", localstruct->state);
        /* do different things based on the state */
        switch (localstruct->state)
        {
                case POP3_CONNECT:
                {
                        isopenretval = is_open(localstruct->filedes);
                        if (debug)
                            printf("is_open is returning %d\n",isopenretval);
                        if ((isopenretval != -1) &&
                                (isopenretval != SYSM_INPROG))
                        {
                                time(&localstruct->lastact);
                                if (debug)
                                        printf("is_open() in service_test_pop3()\n");
                                if (isopenretval != SYSM_OK)
                                {
                                        here->retval = isopenretval;
                                        close(localstruct->filedes);
                                        FREE(localstruct);
                                        here->monitordata = NULL;
                                        if (debug)
                                                printf("ending service_test_pop3() with retval = %s\n", errtostr(here->retval));
                                        return;
                                }
                                localstruct->state = POP3_WAITBAN;
                                if (debug)
                                        printf("connected() to the pop3 port of %s\n", here->checkent->hostname);
                        }
                        break;
                }
                case POP3_WAITBAN:
                {
                        if (data_waiting_read(localstruct->filedes, 0))
                        {
                                memset(buffer, 0, sizeof(buffer));
                                read(localstruct->filedes, buffer, 254);
                                if (debug)
                                        printf("Got :%s:\n", buffer);
                                if (strncmp(buffer, "+OK ", 4) == 0)
                                {
				        /* prepare the buffer */
				        snprintf(buffer, 253, "USER %s", 
						here->checkent->username);

				        /* send the buffer out the socket */
				        sendline(localstruct->filedes, buffer);
                                        localstruct->state = POP3_SENT_USER;
                                } else {
                                        here->retval = SYSM_BAD_RESP;
                                }
                        }
                        break;
                }
		case POP3_SENT_USER:
		{
                        if (data_waiting_read(localstruct->filedes, 0))
                        {
                                memset(buffer, 0, sizeof(buffer));
                                read(localstruct->filedes, buffer, 254);
                                if (debug)
                                        printf("Got :%s:\n", buffer);
                                if (strncmp(buffer, "+OK ", 4) == 0)
                                {
                                        /* prepare the buffer */
                                        snprintf(buffer, 254, "PASS %s", 
						here->checkent->password);

                                        /* send the buffer out the socket */
                                        sendline(localstruct->filedes, buffer);
                                        localstruct->state = POP3_SENT_PASS;
                                } else {
                                        here->retval = SYSM_BAD_RESP;
                                }
                        }
                        break;
		}
		case POP3_SENT_PASS:
		{
                        if (data_waiting_read(localstruct->filedes, 0))
                        {
                                memset(buffer, 0, 256);
                                read(localstruct->filedes, buffer, 254);
                                if (debug)
                                        printf("Got(pop3_sent_pass):%s:\n", 
						buffer);
				if (strncmp(buffer, "+OK ", 4) == 0)
					here->retval = SYSM_OK;
				else if (strncmp(buffer, "-ERR", 4) == 0)
                                {
					here->retval = SYSM_BAD_AUTH;
                                } else {
                                        here->retval = SYSM_BAD_RESP;
                                }
                                sendline(localstruct->filedes, "QUIT");
                                localstruct->state = POP3_SENT_QUIT;
                        }
                        break;
		}
                case POP3_SENT_QUIT:
                {
                        if (data_waiting_read(localstruct->filedes, 0))
                        {
                                memset(buffer, 0, 256);
                                read(localstruct->filedes, buffer, 254);
                                if (debug)
                                        printf("Got2:%s:\n", buffer);
                                if (strncmp(buffer, "+OK ", 4) == 0)
                                        localstruct->state = POP3_GOT_BYE;
                        }
                        break;
                }

	} /* end of switch */
        if (here->retval != -1)
        {
                /* insert cleanup code here */
                close (localstruct->filedes);
                FREE(localstruct);
                here->monitordata = NULL;
        }
        return;


}

void
stop_test_pop3(struct monitorent *here)
{
        struct pop3data *localstruct = NULL;
 
        localstruct = here->monitordata;

        close(localstruct->filedes);
        FREE(localstruct);
        here->monitordata = NULL;
        return;
}

