/* $Id: smtp.c,v 1.8 1997/11/18 05:59:57 jared Exp $ */
#include "config.h"

/* error banners 451 - out of memory */

extern int debug;

/* ok banner 220 */

struct smtpdata {
	int filedes;
	time_t start; /* test start time */
	time_t lastact; /* last activity */
	int state;
	};

#define SMTP_CONNECT	1 /* doing a connect() */
#define SMTP_WAITBAN	2 /* Waiting for the banner */
#define SMTP_SENT_QUIT	3 /* sent a QUIT */
#define SMTP_GOT_BYE	4 /* got a bye from the remote server */

/* 

	Here's the deal with smtp checks.  We do this:

We connect, and do this:

server>220 puck.nether.net ESMTP
us<QUIT
server>221 puck.nether.net closing connection

	If we get anything else, we return  SYSM_BAD_RESP

	If inactivity on socket is more than globtimeout, we return
	SYSM_NORESP

 */

void	start_test_smtp(struct monitorent *here)
{
        struct smtpdata *localstruct = NULL;
        struct my_hostent *hp = NULL;
        struct sockaddr_in name;
        int serrno = -1, errcode = 0;

        /* Allocate our memory */
        here->monitordata = MALLOC(sizeof(struct smtpdata));

        localstruct = here->monitordata;

        time(&localstruct->start);

	localstruct->lastact = localstruct->start;

        localstruct->filedes = open_sock();

        if (localstruct->filedes == -1)
		abort();	/* Bad error, code around it later */

        hp = my_gethostbyname(here->checkent->hostname);

        /* zero out the space */
        memset ( &name, 0, sizeof ( name ) );

        /* copy data */
        memcpy((char*)&name.sin_addr, (char*)hp->my_h_addr, hp->h_length);

        /* set family type */
        name.sin_family = AF_INET;

        /* set the port we're connecting to */
        name.sin_port = htons(here->checkent->port);

        if (debug)
		printf("start_test_smtp() doing connect() to %s:%d\n",
                	here->checkent->hostname, here->checkent->port);

        errcode = connect(localstruct->filedes, (struct sockaddr*)&name,
                sizeof(struct sockaddr_in));
	serrno = errno;
	if (debug)
		perror("connect() in start_smtp");

        if ((errcode < 0) && (serrno != EINPROGRESS))
        {
                if (close(localstruct->filedes) == -1)
                        abort(); /* close the fd incase it's left */
		switch(serrno)
		{
			case ECONNREFUSED:
			case EINTR:
				here->retval = SYSM_CONNREF;
				break;
			case ENETUNREACH:
                        	here->retval = SYSM_NETUNRCH;
				break;
			case EHOSTDOWN:
			case EHOSTUNREACH:
				here->retval = SYSM_HOSTDOWN;
				break;
			case ETIMEDOUT:
				here->retval = SYSM_TIMEDOUT;
		}

	        /* Free memory we'd normally leak */
	        FREE(localstruct);
	        here->monitordata = NULL;

		printf("smtp: retval = %d\n", here->retval);
		return;
        }

	/* doing a connect() */
	localstruct->state = SMTP_CONNECT;

        /* poll it again later */
        return;
}

void	service_test_smtp(struct monitorent *here)
{
	/* do the actual real parts of the checks after 
	   it's been set up */
	
	struct smtpdata *localstruct = NULL;
	char buffer[256];
        int isopenretval = -1;

	/* do some variable shufflign */
	localstruct = here->monitordata;
	if (localstruct == NULL)
	{
		printf("bug! localstruct == NULL in smtp.c:service_test_smtp\n");
		return;
	}

	if (debug)
	printf("in service_test_smtp and the state is %d\n",localstruct->state);

	/* do different things based on the state */
	switch (localstruct->state)
	{
		case SMTP_CONNECT:
		{
			isopenretval = is_open(localstruct->filedes);
		        if ((isopenretval != -1) && 
				(isopenretval != SYSM_INPROG))
		        {
				time(&localstruct->lastact);
		                if (debug)
					printf("is_open() in service_test_smtp()\n");
				if (isopenretval != SYSM_OK)
				{
			                here->retval = isopenretval;
			                close(localstruct->filedes);
			                FREE(localstruct);
			                here->monitordata = NULL;
			                if (debug)
						printf("ending service_test_smtp() with retval = %s\n", errtostr(here->retval));
					return;
				}
		                localstruct->state = SMTP_WAITBAN;
				if (debug)
					printf("connected() to the smtp port of %s\n", here->checkent->hostname);
        		}
			break;
		}
		case SMTP_WAITBAN:
		{
			if (data_waiting_read(localstruct->filedes, 0))
			{
				memset(buffer, 0, 256);
				read(localstruct->filedes, buffer, 254);
				buffer[strlen(buffer)-1] = '\0';
				if (debug)
					printf("Got :%s:\n", buffer);
				if (strncmp(buffer, "220 ", 4) == 0)
				{
					localstruct->state = SMTP_SENT_QUIT;
				} else {
					here->retval = SYSM_BAD_RESP;
				}
				sendline(localstruct->filedes, "QUIT");
			}
			break;
		}
		case SMTP_SENT_QUIT:
		{
                        if (data_waiting_read(localstruct->filedes, 0))
                        {
                                memset(buffer, 0, 256);
                                read(localstruct->filedes, buffer, 254);
                                buffer[strlen(buffer)-1] = '\0';
                                if (debug)
					printf("Got2:%s:\n", buffer);
				if (strncmp(buffer, "221 ", 4) == 0)
					localstruct->state = SMTP_GOT_BYE;
				else
					here->retval = SYSM_BAD_RESP;
                        }
                        break;
		}
		case SMTP_GOT_BYE:
		{
			here->retval = SYSM_OK;
			if (debug)
				printf("smtp: returing a OK value\n");
			break;
		}
		default:
		{
			abort(); /* Muahaha */
			break;
		}
	}
	if (here->retval != -1)
	{
		/* insert cleanup code here */
		close (localstruct->filedes);
		FREE(localstruct);
		here->monitordata = NULL;
	}
	return;
}

void
stop_test_smtp(struct monitorent *here)
{
        struct smtpdata *localstruct = NULL;
 
        localstruct = here->monitordata;
	if (localstruct == NULL)
		return;

        close(localstruct->filedes);
        FREE(localstruct);
        here->monitordata = NULL;
        return;
}

