/* $Id: srvclient.c,v 1.23 1997/12/05 21:59:46 jared Exp $ */
#include "config.h"

extern struct hostinfo *currenthead;
extern struct clientstatus *clienthead;
extern int inactivetime;
extern int boottime; /* does a time() when program starts */
extern int debug; /* like a debug flag */
extern int numqueued; /* num of elements in the queue */

int inhere = 0;

void client_poll();

void send_stat_start(struct clientstatus client, struct hostinfo *here)
{
	char buff[256];

	if (here == NULL)
		return; /* shouldn't happen, just doing sanity checking */

	if (here->lastcheck != 0)
	{
		/* down in some fashion */
		sprintf(buff, "%s:%d:%d:%d:%d:%d:%ld", here->hostname,
			here->type, here->port, here->lastcheck,
			here->downct, here->contacted, here->deathtime);

		/* write it out the socket */
		sendline(client.filedes, buff);
	
		/* stat of siblings */
                send_stat_start(client, here->sibling);
	} 
	else 
	{
		if (here->child != NULL)
			/* stat of the kids */
			send_stat_start(client, here->child);
		if (here->sibling != NULL)
			/* stat of the siblings */
			send_stat_start(client, here->sibling);
	}
	return;
}

int	send_stat(struct clientstatus client)
{
	send_stat_start(client, currenthead);
	if (sendline(client.filedes, "333") == -1)
	{
		syslogmsg("unable to send message to client\n");
		client.filedes = -1;
		return -1;
	}

	return 0;
}

void	send_conf_det(int cli, struct hostinfo *here)
{
	char buffer[256];

	if (here == NULL)
		return;

	sprintf(buffer, "%s %s %d %s %s %s %s %s %d %d qud:%d",
		here->hostname, type_to_name(here->type), here->port,
		here->message, here->contact, errtostr(here->lastcheck), 
		here->username, here->password, here->downct, 
		here->contacted, here->queued);

	if (here->child != NULL)
		strcat(buffer, " {");

	sendline(cli, buffer);
	
	send_conf_det(cli, here->child);

	if (here->child != NULL)
		sendline(cli, "}");

	send_conf_det(cli, here->sibling);
}

int	send_conf(struct clientstatus *client)
{
	send_conf_det(client->filedes, currenthead);

	/* done printing config */
	if (sendline(client->filedes, "333") == -1)
	{
		syslogmsg("unable to send message to client\n");
		return -1;
	}
	return 0;
}

int	do_upd(struct clientstatus *client, char *buff)
{
	if (sendline(client->filedes, "444") == -1)
	{
		syslogmsg("unable to send message to client\n");
		return -1;
	}
	return 0;
}

int	do_auth(struct clientstatus *client, char *buff)
{
        if (sendline(client->filedes, "444") == -1)
        {
                syslogmsg("unable to send message to client\n");
                return -1;
        }
        return 0;
}

void send_uptime(struct clientstatus *client)
{
	char buffer[256];

	sprintf(buffer, "Uptime = %s", str_difftime_sec(boottime));
	sendline(client->filedes, buffer);
}

/* CLIENT managment code */
void	do_service(struct clientstatus *here, char *buff)
{
	/* Local buffer */
	char lb[250];

	if (strncmp(buff, "STAT", 4) == 0)
	{
		send_stat(*here);
	}
	else if (strncmp(buff, "UPTIME", 6) ==0)
	{
		send_uptime(here);
	}
	else if (strncmp(buff, "NFD", 3) == 0)
	{
		sprintf(lb,"%d is the next FD- %d queued", nextfd(), numqueued);
		sendline(here->filedes, lb);
		sendline(here->filedes, "what do you think about that?");
	}
	else if (strncmp(buff, "ABORT", 5) == 0)
	{
		/* dump a core or something */
		abort();
	}
	else if (strncmp(buff, "KILLIT", 6) == 0)
	{
		stop_daemon();
	}
	else if (strncmp(buff, "QUIT", 4) == 0)
	{
		sendline(here->filedes, "333 Good Bye, please come again");
		close(here->filedes);
		here->filedes = -1;
	}
	else if (strncmp(buff, "DEBUG", 5) == 0)
	{
		/* Toggle Debugging */
		if (debug == 1)
			debug = 0;
		else 
			debug = 1;
		sendline(here->filedes, "Toggled debugging");
		syslogmsg("Toggled debugging at client request");
	}
	else if (strncmp(buff, "CONF", 4) == 0)
	{
		send_conf(here);
	}
	else if (strncmp(buff, "EXPIREDNS", 9) == 0)
	{
		expire_dns();
	}
	else if (strncmp(buff, "PRINTQ", 6) == 0)
	{
		print_queue(here->filedes);
	}
	else if (strncmp(buff, "VERS", 4) == 0)
	{
		sendline(here->filedes, VERS);
	}
	else if (strncmp(buff, "AUTH", 4) == 0)
	{
		do_auth(here, buff);
	}
	else if (strncmp(buff, "UPD", 3) == 0)
	{
		do_upd(here, buff);
	}
	else
		if (sendline(here->filedes, "444 - Unk")  == -1)
		{
			syslogmsg("error sending message to client\n");
			close(here->filedes);
			here->filedes = -1;
		}
}

void timeout_clients()
{
	struct clientstatus *here;
	time_t now;

	time(&now);
	
	if (clienthead == NULL)
		return;

	for (here = clienthead->next; here != NULL ; here = here->next)
		if ((now - here->lastactivity) > inactivetime )
		{
			sendline(here->filedes, "444 - Timed out");
			if (here->filedes != -1)
				if (close(here->filedes) == -1)
					perror("waah! closing stuff\n");
			here->filedes = -1;
		}
}

void	dead_client_cleanup()
{
	/* free clients with filedes = -1 */

	struct clientstatus *here, *last, *freeme;

	if (clienthead == NULL)
		return;

	last = clienthead;
	freeme = NULL;
        for (here = clienthead->next; here != NULL ; here = here->next)
        {
		if (freeme != NULL)
		{
			FREE(freeme);
			freeme = NULL;
		}
		
		if (here->filedes == -1)
		{
			freeme = here;	
			last->next = here->next;
		} else {
			last = here;
		}
	}
	if (freeme != NULL)
	{
		FREE(freeme);
		freeme = NULL;
	}
}

void	setup_client()
{
        int msgsock; /* message socket for talking back and forth */
        char buff[256]; /* buffer for formatting text */
        struct sockaddr_in remote;
        int size;
	struct clientstatus *thisclient, *linkafter;

        size = sizeof (remote );

        /* do the accept() on the socket, then take care of it */
        msgsock = accept(clienthead->filedes,(struct sockaddr *)&remote,&size);

        if (msgsock == -1)
        {
                perror("accept");
                return;
        }

        /* figure out who's connecting and log it */
        sprintf(buff, "accepting connection from %s\n", 
		inet_ntoa(remote.sin_addr));

	/* log the message */
        syslogmsg(buff);

        /* print our greeting banner */
        if (debug)
                syslogmsg("sending hello banner\n");

        sendline(msgsock, "111 - v1.0 Ready");

	if (debug)
		syslogmsg("setting up client structure");

	thisclient = MALLOC(sizeof(struct clientstatus));

	memset(thisclient, 0, sizeof(struct clientstatus));

	time(&thisclient->lastactivity); /* set current time */
	thisclient->filedes = msgsock; /* save accept()'ed fd */
	strcpy(thisclient->ip, inet_ntoa(remote.sin_addr)); /* save source ip */
	thisclient->authlvl = 0; /* no auth yet */
	thisclient->clientver = 0; /* no client vers yet, sent ours */
	thisclient->next = NULL; /* next client = NOTHING */

	linkafter = clienthead; /* start to walk client list */

	while (linkafter->next != NULL) /* until next client is null */
		linkafter = linkafter->next; /* do walk */
	linkafter->next = thisclient; /* link us at the end of it */
}

void	check_for_new_clients()
{
	/* check the specified socket for awating connections, if so, take
	   care of them */
	fd_set rd, wr, except;
	struct timeval local_timeout;
	
	if (clienthead->filedes == -1)
	/* do nothing */
		return;

	local_timeout.tv_sec = 0;
	local_timeout.tv_usec = 0;

	FD_ZERO(&wr);
	FD_ZERO(&except);
	FD_ZERO(&rd);
	FD_SET(clienthead->filedes, &rd);

	if (debug)
		syslogmsg( "Checking for a client\n");

	while (select(clienthead->filedes+1,&rd,&wr,&except,&local_timeout)!=0)
		setup_client();

}

void	service_clients()
{
	struct clientstatus *here;
	char throwmeout[256], buff[256];
	int did_service = 1;
	
	while  (did_service == 1)
	{
		did_service = 0;
		for (here=clienthead->next;here!=NULL;here=here->next)
		{
			if (here->filedes == -1)
				continue;
			if (data_waiting_read(here->filedes, 0) != 0)
			{
				
				getline_tcp(here->filedes, buff);
				getline_tcp(here->filedes, throwmeout);
				did_service = 1;
				do_service(here, buff);
				time(&here->lastactivity);
			}
		}
	}


}

void	client_poll()
{
        fd_set rd, wr, except;
        struct timeval local_timeout;
	struct clientstatus *here;

	if (inhere)
		return;
	inhere = 1;

        if (clienthead->filedes == -1)
        /* do nothing */
                return;

        local_timeout.tv_sec = 0;
        local_timeout.tv_usec = 0;

        FD_ZERO(&wr);
        FD_ZERO(&except);
        FD_ZERO(&rd);

	/* check for new clients */
	check_for_new_clients();

	/* process old clients new data */
	service_clients();

	/* timeout old clients */
	timeout_clients();

	/* free dead client memory */
	dead_client_cleanup();

	for (here = clienthead; here != NULL; here = here->next)
		if (here->filedes != -1)
	        	FD_SET(here->filedes, &rd);

	if (select(clienthead->filedes+1,&rd,&wr,&except,&local_timeout) != 0)
	{
	        check_for_new_clients();
		service_clients();
	}

	inhere = 0;
}
