/* $Id: syswatch.c,v 1.93 1998/01/14 11:20:20 jared Exp $ */
#include "config.h"

int lvl = 0; /* internal counter for the braces */
int ln = 0; /* line number */
char configfile[256]; /* config file path */
int numfailures = 4; /* Number of failures before mailing contact */
int inactivetime = 300; /* timeout for client inactivity */
int globtimeout = 30; /* amount of time it takes to time out internal tests 
			(in seconds) */
int globtimeoutlen = 35; /* when to dequeue the request (in seconds) */
int donotify = 0; /* Do notifies - a value of 1 doesn't notify contacts */
int pageinterval = 0; /* no re-page interval for hosts when they're down,
			otherwise the number of minutes to re-page in */
int dofork = 0; /* 0 = fork, 1 = don't */
int showupalso = 0; /* show up also in textfile */
int ckconfigonly = 0; /* 0 = normal, 1 = just check config then exit */
int numqueued = 0;   /* number of elements in the queue */
int maxqueued = 0;   /* number of elements that can be in the queue at
			once - delay queueing until it comes down,
			default = 0 = Unlimited */
time_t boottime = 0; /* boot time for uptime */
int gotsighup = 0;   /* quit all things we're doing asap, and then
			cleanup and shuffle config files, etc.. */
int dnslog = 600;
int dnsexpire = 900;
int heartbeat = 1;
unsigned short int killafter = 45; /* kill a check after this many seconds
	in the queue with no return value */
char *errorsto = NULL;
char *myname; /* myname */
int debug = 0; /* debugging flag - 0 = off 1 = on */
int mallocdebug = 0; /* 0 = off, 1 = on */
struct hostinfo *currenthead = NULL; /* something */
struct dnscache *dnshead = NULL;
struct clientstatus *clienthead;
struct monitorent *queuehead = NULL;
struct protoent *icmpproto;
int facility = LOG_DAEMON;
int html = -1;
unsigned int statuschanged = 0;
char *statusfilename = NULL;

void usage ()
{
	/* Print out the real way to use the program, morons! */
	fprintf(stderr,"Usage: %s[ -f config-file ] [ -n ]\n", myname);
	fprintf(stderr,"  -f config-file : Alternate config file location\n");
	fprintf(stderr,"  -n             : Don't do notifies\n");
	fprintf(stderr,"  -d             : Don't fork\n");
	fprintf(stderr,"  -v             : Print version then exit\n");
	fprintf(stderr,"  -t             : Test/check config file then exit\n");
	fprintf(stderr,"  -p #           : Change port number listening on\n");
	exit(1);
}

/* parse the command line arguments */
void cmdline(int argc, char **argv, char *conf_file, int *listenport)
{
	int er = 0, x;

	for (x=1; argc > x ; x++)
	{
		if (argv[x][0]=='-')
			switch (argv[x][1]) 
			{
				case 'f':
				{
					if (x+1 == argc)
						usage();
					strcpy(conf_file,argv[x]+2);
					if (strlen(conf_file) == 0)
					{
						strcpy(conf_file,argv[x+1]);
						x++;
					}
					break;
				}
				case 'p': /* alternate port number */
				{
					if (x+1 == argc)
						usage();
					*listenport = atoi(argv[x+1]);
					x++; /* skip it as the next arg */
					break;
				}
				case 'n': /* don't do notifies */
				{
					donotify = 1;
					break;
				}
				case 'd': /* don't fork */
				{
					dofork = 1;
					break;
				}
				case 't': /* verify config file */
				{
					ckconfigonly = 1;
					break;
				}
				case 'M': /* malloc debug on */
				{
					mallocdebug=1;
					break;
				}
				case 'D': /* debug on */
				{
					fprintf(stderr, "Debug on\n");
					debug = 1;
					break;
				}
				case 'v': /* print version */
				{
					fprintf(stdout, "%s\n", VERS);
					exit(0);
				}
				case '-': /* gnu style args */
				{
					if (strcmp(argv[x]+2, "version") == 0)
					{
						fprintf(stdout, "%s\n", VERS);
						exit(0);
					}
					if (strcmp(argv[x]+2, "help") == 0)
						usage();
				}
				default:
					er++;
			}
	}
	if (er > 0)
		usage();
}

void stop_daemon()
{
	char syslogbuf[256]; /* internal buffer for formatting messages */

	fprintf(stderr, "Quitting due to deadly signal\n");
	/* insert cleanup code here */

	/* dump current network status to a file */
	if (html != -1)
		dump_to_file(statusfilename, html);

	/* log shutdown message */
	sprintf(syslogbuf, "shutting down due to deadly signal");

	/* timeout old clients */
	inactivetime = 0;
	timeout_clients();

	/* free dead client memory */
	dead_client_cleanup();

	if (clienthead != NULL)
	{
		/* un bind the listen socket */
		close(clienthead->filedes); 
	}

	/* free the memory in use */
	free_tree(currenthead);

	syslogmsg( syslogbuf);

	exit(1);
}

void setup_client_listen(int listenport)
{
	clienthead = MALLOC(sizeof(struct clientstatus));

	/* listen on port PORTNUM */
	clienthead->filedes = init_socket(listenport);

	/* master client socket */
	clienthead->clientver = -1;
}

void recurs_print_q(int filedes, struct hostinfo *here)
{
	char buffer[256];

	if (here == NULL)
		return;

	if (here->queued == 1)
	{
		sprintf(buffer, "treeQ: %s:%d", here->hostname, here->type);
	       	sendline(filedes, buffer);
	}
	recurs_print_q(filedes, here->child);
	recurs_print_q(filedes, here->sibling);
}

void print_queue(int filedes)
{
	struct monitorent *here;
	char buffer[256];

	/* Print out the currently queued entries for debugging */
	for (here = queuehead; here!= NULL; here = here->next)
	{
		sprintf(buffer, "Queue: %s:%s:%d", here->checkent->hostname,
			type_to_name(here->checkent->type),
			here->checkent->port);
		sendline(filedes, buffer);
	}
	sendline(filedes, "End of Queue based on queuehead");

	recurs_print_q(filedes, currenthead);

	sprintf(buffer, "Number of elements in queue count: %d", numqueued);
	sendline(filedes, buffer);

	return;
}

void
queue_check(struct hostinfo *entry)
{
	struct monitorent *newentry = NULL;

	/* put a pointer on the stack for this entry */

	/* stick it at the top for easy access */

	if (entry == NULL)
	{
		printf("queue_check called w/ NULL\n");
		return;
	}
	if (entry->queued == 1)
	{
		printf("queue_check: Attempt to queue check already in q\n");
		return;
	}
	newentry = MALLOC(sizeof(struct monitorent));

	newentry->checkent = entry;
	newentry->started = 0;
	newentry->monitordata = NULL;
	gettimeofday(&newentry->queueat, NULL);
	newentry->retval = -1;
        newentry->checkent->queued = 1;

	/* insert it at the top of the queue */
        newentry->next = queuehead;
	queuehead = newentry;

	numqueued++;

	/* entry is queued for checking */
	if (debug)
	{
		fprintf(stderr, "queue_check:Queued %s:%s:%p\n", 
			entry->hostname, type_to_name(entry->type), entry);
	}

	return;
}

/*      Routine used to do recursive walking of the n-ary
	tree to check entries for their queue status */
void recurs_queue_checks(struct hostinfo *entry)
{
	if (entry == NULL)
	{
		printf("bug! recurs_queue_checks called w/ NULL arg\n");
		return;
	}
	if (entry->queued == 0) 
	{
		if ((entry->lchecktime+60) < time(NULL))
		{
			if ((maxqueued == 0) || (numqueued >= maxqueued))
			{
				queue_check(entry);
			}
		} else if ((entry->queued == 0) && (entry->lastcheck == 0) && 
				(entry->child != NULL)) 
		{
			recurs_queue_checks(entry->child);
		}
	}

	if (entry->sibling != NULL)
		recurs_queue_checks(entry->sibling);
}

void queue_checks()
{
	/* walk n-ary tree and queue checks that need it */
	recurs_queue_checks(currenthead);
}

void needssleep()
{
	fd_set rd, wr, except;
	struct timeval local_timeout;
	struct clientstatus *clienthere;
	int mincalctime = 1, maxfd = 1;

	/* determine if we need sleep */

	/* set up all the data structures */
	FD_ZERO(&wr);
	FD_ZERO(&except);
	FD_ZERO(&rd);

	/* Watch the clients for data to be read */

	for (clienthere = clienthead->next;
		clienthere != NULL;
		clienthere = clienthere->next)
	{
		if (clienthere->filedes == -1)
			continue;
		FD_SET(clienthere->filedes, &rd);
		if (clienthere->filedes > maxfd)
			maxfd = clienthere->filedes;
	}
	
	/* set the clienthead */
	FD_SET(clienthead->filedes, &rd);

	/* watch for data to read on this fd */
	/*              FD_SET(fd, &rd); */

	/* watch for the ability to write on this fd */
	/*              FD_SET(fd, &wr); */

	local_timeout.tv_sec = mincalctime;
	local_timeout.tv_usec = 0;

	select(maxfd+1, &rd, &rd, &except, &local_timeout);

	/* basically, set up all the fd's we need to watch
		in a fd_set with select timeout set for the
		minimum time for any check in the tree for
		the time it needs to wake up, include client
		fd's in this also */
}

void pagefolks()
{
	/* page people that just went down */
}

void service_this(struct monitorent *here)
{
	char buffer[256];

	if (here == NULL)
	{
		syslogmsg("BUG!  service_this() called with here== NULL");
		return;
	}
	if (debug)
	{
		sprintf(buffer, "Servicing entry in queue of %s:%s\n",
			here->checkent->hostname,
			type_to_name(here->checkent->type));
		syslogmsg(buffer);
	}

	if (here->started)
	{
		/* service a check already started */
		switch (here->checkent->type)
		{
			case SYSM_TYPE_TCP: service_test_tcp(here);
				break;
			case SYSM_TYPE_UDP: service_test_udp(here);
				break;
                        case SYSM_TYPE_PING: service_test_ping(here);
                                break;
#ifdef IMPLEMENT
                        case SYSM_TYPE_SNMP: service_test_snmp(here);
                                break;
#endif /* IMPLEMENT */
                        case SYSM_TYPE_NNTP: service_test_nntp(here);
                                break;
                        case SYSM_TYPE_SMTP: service_test_smtp(here);
                                break;
                        case SYSM_TYPE_IMAP: service_test_imap(here);
                                break;
                        case SYSM_TYPE_POP3: service_test_pop3(here);
                                break;
                        case SYSM_TYPE_X500: service_test_x500(here);
                                break;
#ifdef IMPLEMENT
                        case SYSM_TYPE_POP2: service_test_pop2(here);
                                break;
                        case SYSM_TYPE_BOOTP: service_test_bootp(here);
                                break;
                        case SYSM_TYPE_DNS: service_test_dns(here);
                                break;
#endif /* IMPLEMENT */
                        case SYSM_TYPE_WWW: service_test_www(here);
                                break;
#ifdef IMPLEMENT
                        case SYSM_TYPE_RADIUS: service_test_radius(here);
                                break;
#endif /* IMPLEMENT */
#ifdef HAVE_SSL
                        case SYSM_TYPE_HTTPS: service_test_https(here);
                                break;
#endif /* HAVE_SSL */
			case SYSM_TYPE_SYSM:
				break;

			default: abort();
				break;
                } /* end switch */
	}
	if (!here->started)
	{
		switch (here->checkent->type)
		{
			case SYSM_TYPE_TCP: 
				start_test_tcp(here);
				break;
			case SYSM_TYPE_UDP: 
				start_test_udp(here);
				break;
			case SYSM_TYPE_PING: 
				start_test_ping(here);
				break;
#ifdef IMPLEMENT
			case SYSM_TYPE_SNMP: 
				start_test_snmp(here);
				break;
#endif /* IMPLEMENT */
			case SYSM_TYPE_NNTP: 
				start_test_nntp(here);
				break;
			case SYSM_TYPE_SMTP: start_test_smtp(here);
				break;
			case SYSM_TYPE_IMAP: start_test_imap(here);
				break;
			case SYSM_TYPE_POP3: start_test_pop3(here);
				break;
			case SYSM_TYPE_X500: start_test_x500(here);
				break;
#ifdef IMPLEMENT
			case SYSM_TYPE_POP2: start_test_pop2(here);
				break;
			case SYSM_TYPE_BOOTP: start_test_bootp(here);
				break;
			case SYSM_TYPE_DNS: start_test_dns(here);
				break;
#endif /* IMPLEMENT */
			case SYSM_TYPE_WWW: start_test_www(here);
				break;
#ifdef IMPLEMENT
			case SYSM_TYPE_RADIUS: start_test_radius(here);
				break;
#endif /* IMPLEMENT */
#ifdef HAVE_SSL
			case SYSM_TYPE_HTTPS: start_test_https(here);
				break;
#endif /* HAVE_SSL */
			case SYSM_TYPE_SYSM:
				break;

			default: abort();
				break;
		} /* end switch */
		here->started = TRUE;
	}
}

void handle_retval(struct monitorent *handle_this)
{
	if (debug)
	{
		printf("handle_retval: dequeued %s:%s:%p\n",
			handle_this->checkent->hostname,
			type_to_name(handle_this->checkent->type),
			handle_this);

		printf("handle_retval:checking %s type %s got us %s\n",
			handle_this->checkent->hostname,
			type_to_name(handle_this->checkent->type),
			errtostr(handle_this->retval));
	} /* endif debug */

	/* Do necessary post-check status stuff */

	/* check to see if the service came up */
	if ((handle_this->retval == 0) &&
		(handle_this->checkent->contacted == TRUE) &&
		(handle_this->checkent->lastcheck != handle_this->retval))
	{
		/* if so, set the state */
		handle_this->checkent->lastcheck = handle_this->retval;
		/* determine if we need to page */
		if (donotify == 0)
		{
			/* page */
			page_someone(handle_this->checkent);
		}
		time(&handle_this->checkent->last_up);
		/* reset contacted */
		handle_this->checkent->contacted = FALSE; /* init it */
		statuschanged=1;
	} /* endif some long ass thing */

	/* if service is down, and state is
	   same as last time */
	if ((handle_this->retval != 0) &&
		(handle_this->checkent->lastcheck == handle_this->retval))
	{
		/* increment times service has failed */
		handle_this->checkent->downct++;

		/* if it's failed too many times and we
			haven't paged */

		if ((handle_this->checkent->downct >=
			handle_this->checkent->max_down) &&
			(handle_this->checkent->contacted == FALSE))
		{
			/* page someone */
			page_someone(handle_this->checkent);
		}
		statuschanged=1;
	} /* endif some other retval check thing */

	/* if state changed but still failing */
	if (handle_this->retval != handle_this->checkent->lastcheck)
	{
		/* reset stuff */
		handle_this->checkent->downct = 1;
		handle_this->checkent->lastcheck = handle_this->retval;
		time(&handle_this->checkent->deathtime);
		statuschanged=1;
	} /*endif handle_this->retval!=handle_this->checkent->lastcheck*/

	return;
}

void service_checks()
{
	struct monitorent *here, *last = NULL, *freeit = NULL;

	if (queuehead == NULL)
		return;

	/* setup checks that need to be, service current checks */
	here = queuehead;
	while (here != NULL)
	{
		/* if no return value */
		if (here->retval == -1)
			service_this(here); /* service the check */
		if (here->retval != -1)
		{
                        /* Handle the return value */
                        handle_retval(here);

			/* set the current element to be free'ed */
			freeit = here;

			if (here->checkent == NULL)
			{
				printf("bug! service_checks: here->checkent == NULL\n");
				abort();
			}
                        here->checkent->queued=0; /* tag other element as non-queued */
			numqueued--; 	/* decrease queue counter */

			time(&(here->checkent->lchecktime)); /* save the time */

			if (last == NULL)
			{ /* If we intend to free the head of the queue */
				queuehead = freeit->next;
			} else {
				last->next = here->next;
			}
		} else  {
			last = here;
		}
		 
		here = here->next;

	        if (freeit != NULL)
	        {
	                FREE(freeit);
			freeit = NULL;
	        }
	} /* end while */

	/* dump current network status to a file */
	if ((html != -1) && statuschanged)
	{
		dump_to_file(statusfilename, html);
	}
}

void fast_cleanup_checks()
{
	struct monitorent *here, *next;
	/* cleanup checks, and shut them down asap */

	if (numqueued == 0)
	{
		if (debug)
		{
			printf("fast_cleanup_checks w/ no checks in queue\n");
		}
		return;
	} 

	/* walk the current queue */
        for (here = queuehead ; here != NULL ; here=here->next)
	{
		if (debug) {
		printf("in q @ sighup - %s:%d\n", here->checkent->hostname,
			here->checkent->type);
		}
		if (!here->started)
		{
			here->checkent->queued = 0;
			here->retval = here->checkent->lastcheck;
		} else {
			if (here->monitordata != NULL)
			switch (here->checkent->type)
			{
				case SYSM_TYPE_TCP:
					stop_test_tcp(here);
					break;
				case SYSM_TYPE_UDP:
					stop_test_udp(here);
					break;
				case SYSM_TYPE_PING:
					stop_test_ping(here);
					break;
#ifdef IMPLEMENT
				case SYSM_TYPE_SNMP:
					stop_test_snmp(here);
					break;
#endif /* IMPLEMENT */
				case SYSM_TYPE_NNTP:
					stop_test_nntp(here);
					break;
				case SYSM_TYPE_SMTP:
					stop_test_smtp(here);
					break;
				case SYSM_TYPE_IMAP:
					stop_test_imap(here);
					break;
				case SYSM_TYPE_POP3:
					stop_test_pop3(here);
					break;
				case SYSM_TYPE_X500:
					stop_test_x500(here);
					break;
#ifdef IMPLEMENT 
				case SYSM_TYPE_POP2:
					stop_test_pop2(here);
					break;
				case SYSM_TYPE_BOOTP:
					stop_test_bootp(here);
					break;
				case SYSM_TYPE_DNS:
					stop_test_dns(here);
					break;
#endif /* IMPLEMENT */
				case SYSM_TYPE_WWW:
					stop_test_www(here);
					break;
#ifdef IMPLEMENT
				case SYSM_TYPE_RADIUS:
					stop_test_radius(here);
					break;
				case SYSM_TYPE_HTTPS:
					stop_test_https(here);
					break;
				case SYSM_TYPE_SYSM:
					stop_test_sysm(here);
					break;
				case SYSM_TYPE_SSHD:
					stop_test_sshd(here);
					break;
#endif /* IMPLEMENT */
				default:
					abort();
					break;
			}
				else
			{
				printf("bug - monitordata!= null & queued -- retval = %d\n", here->retval);
			}
			
		}
	}

	/* Free everything in the queue */
        for (here = queuehead ; here != NULL ; )
        {
		if (here == NULL)
			break;

		next = here->next;
		FREE(here);
		numqueued--;
		here = next;

	}
	queuehead = NULL;
}

void wakeup_checks()
{
	/* wake up checks that need timeout */
        struct monitorent *here = NULL;
	struct timeval now;
	unsigned int count;
	char buffer[256];

	count = 0;
	gettimeofday(&now, NULL); /* get the current time */

	if (debug)
	{
		printf("wakeup_checks() waking up checks\n");
	}
        /* walk the current queue */
	here = queuehead;
        while (here != NULL)
        {
		if (count > (numqueued +10))
		{
			printf("wakeup_checks() BUG! bailing out of while() as count is too hight\n");
			break;
		}
		if (debug)
		{
			printf("wakeup_checks() looking at %s:%s:%d\n", 
                                here->checkent->hostname,
                                type_to_name(here->checkent->type),
                                here->checkent->port);
		}
		if (mydifftime(here->queueat, now) >= killafter)
		{
                        sprintf(buffer, "Killing stale check %s:%s:%d lasting %10.6fsecs",
                                here->checkent->hostname,
                                type_to_name(here->checkent->type),
                                here->checkent->port,
                                mydifftime(here->queueat, now) );
                        syslogmsg(buffer);
			if (debug)
				printf("%s\n", buffer);

			here->retval = SYSM_KILLED;
		} else if (mydifftime(here->queueat, now) >= 15.0) {
                        sprintf(buffer, "Possibly stale check of %s:%s:%d lasting %10.6f",
                                here->checkent->hostname,
                                type_to_name(here->checkent->type),
                                here->checkent->port,
                                mydifftime(here->queueat, now) );
                        syslogmsg(buffer);
                        if (debug)
                                printf("%s\n", buffer);
                }
		here = here->next;
		count++;
		if (debug)
		{
			printf("wakeup_checks() - just moved here\n");
		}
	}
	if (debug)
	{
		printf("wakeup_checks() exiting\n");
	}
}

void 
do_new_watch(char *cmdname, int listenport)
{
	time_t one=time(NULL),two;
	struct hostinfo *hupdata = NULL;

	setup_client_listen(listenport);

	while (1)
	{
		queue_checks();   /* queue checks ready for us */
		two = time(NULL);
		check_runtime(one, two, "queue_checks");
		one=two;

                client_poll();    /* check for clients */
                two = time(NULL);
                check_runtime(one, two, "client_poll");
                one=two;

		wakeup_checks();  /* wakeup any "stale" checks */
                two = time(NULL);
                check_runtime(one, two, "wakeup_checks");
                one=two;

                client_poll();    /* check for clients */
                two = time(NULL);
                check_runtime(one, two, "client_poll");
                one=two;

		service_checks(); /* do checks that are ready for us */
                two = time(NULL);
                check_runtime(one, two, "service_checks");
                one=two;

                client_poll();    /* check for clients */
                two = time(NULL);
                check_runtime(one, two, "client_poll");
                one=two;

		needssleep();     /* if we need a sleep, do it here */
                two = time(NULL);
                check_runtime(one, two, "needssleep");
                one=two;

		client_poll();    /* check for clients */
                two = time(NULL);
                check_runtime(one, two, "client_poll");
                one=two;


		pagefolks();	  /* page folks/e-mail them */
                two = time(NULL);
                check_runtime(one, two, "pagefolks");
                one=two;


		do_tree_periodic(); /* Do periodic checks/tests */
                two = time(NULL);
                check_runtime(one, two, "do_tree_periodic");
                one=two;

		if (gotsighup)
		{
			/* Kill all checks currently pending */
			fast_cleanup_checks();

			/* do all the fun stuff */
                        if (debug)
                                syslogmsg("calling sync_after_sighup");
                        hupdata = sync_after_sighup(currenthead, configfile);

                        currenthead = hupdata;

                        if (debug)
                                syslogmsg("resetting gotsighup");
                        gotsighup = 0;
		}

	}
}

void periodic_rewalk(struct hostinfo *here)
{
	if (here == NULL)
		return;
	
	if (here->lastcheck == SYSM_NODNS)
	{
		/* redo dns query */
		if (my_gethostbyname(here->hostname) != NULL)
			here->lastcheck = 0;
	}

	periodic_rewalk(here->child);
	periodic_rewalk(here->sibling);
}

void periodic_page(struct hostinfo  *here)
{
	if (here == NULL)
		return;
	if (here->contacted)
	{
		if ((time(NULL) - here->lastcontacted) > (pageinterval*60))
			page_someone(here);
	}
	periodic_page(here->child);
	periodic_page(here->sibling);
}


void do_tree_periodic()
{
	/* walk tree and check for now valid dns entries */
	periodic_rewalk(currenthead);

	/* walk tree and check for time to repage about sites */
	if (pageinterval != 0)
	{
		periodic_page(currenthead);
	}
}


/* The main part of the program */
int
main (int argc,char **argv)
{
	int pid = 0; /* pid of child */
	int listenport = PORTNUM; /* (default) port to listen on */
	char myhostname[80]; /* my hostname */
        struct rlimit thislimit; /* used for resetting fd limits */
	char syslogbuf[256]; /* used locally when booting */


	boottime = time(NULL);
	srand(boottime); /* Seed the random number generator */

	myname = argv[0];
	currenthead = NULL; /* initalize it */

	gethostname(myhostname, 80); /* get my hostname */

	/* trap those signals that tell us to stop and do proper
	   cleanup */
	signal(SIGHUP, reload_config);
	signal(SIGINT, stop_daemon);
	signal(SIGQUIT, stop_daemon);
	signal(SIGTERM, stop_daemon);

        thislimit.rlim_cur = 1024;
        thislimit.rlim_max = 1024;

#if (defined(__FreeBSD) || defined(FreeBSD) || defined(__FreeBSD__) || \
	defined(sun))

        setrlimit(RLIMIT_NOFILE, &thislimit);
#else
        setrlimit(RLIMIT_OFILE, &thislimit);
#endif

	/* do some setting up for icmp stuff */
        if (!(icmpproto = getprotobyname("icmp")))
        {
                /* huh, what's icmp? */
                fprintf(stderr, "syswatch.c: unknown protocol icmp.\n");
                exit(1);
        }


	strcpy(configfile, CFILE);

	if (argc > 1)
	{
		cmdline(argc, argv, configfile, &listenport);
	}

	currenthead = loadconfig(configfile);

	if (currenthead == NULL)
	{
		syslogmsg("main: currenthead == NULL - nothing to monitor");
	}

	if (ckconfigonly == 1)
		exit(0);

	/* if we're syslogging stuff, log a bootup message in syslog
	 */
	sprintf(syslogbuf, "booting of %s version %s", argv[0], VERS);
	syslogmsg(syslogbuf);

	fprintf(stdout, "%s\n", VERS);
	fprintf(stdout, "%s started on %s\n", myname, myhostname);

	if (heartbeat)
		send_heartbeat(myhostname);

	if (dofork == 0)
		pid = fork();

	if ((pid == 0) || (dofork == 1))
	{
		do_new_watch(argv[0], listenport);
	} else {
		fprintf(stdout, "forked process as pid %d\n", pid);
	}
	exit(0);
}



int
init_socket(int listenport) {

	struct sockaddr_in name;
	int sock;
	int len;

	int on = 1;
	sock = socket(AF_INET, SOCK_STREAM, 0);
	if (sock < 0) {
		perror("opening datagram socket");
		exit(2);
	}

	len = sizeof(on);
	/* use keepalives to force us to die when the remote dies */
	if (setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE, (char*)&on, (int)len) == -1)
	{
	  	perror("init_socket:setsockopt: cannot set socket option");
	}

	name.sin_family = AF_INET;
	name.sin_addr.s_addr = INADDR_ANY;
	name.sin_port = htons(listenport);

	while (1)
	{
		/* bind it */
		if (bind(sock, (struct sockaddr*)&name, 
			sizeof(struct sockaddr_in)) < 0) 
		{
			perror("bind()");
			exit(1);
		} else  {
			break;
		}
	}
	listen(sock,2);
	return sock;

}


