/* $Id: tcp.c,v 1.11 1997/12/02 17:13:46 jared Exp $ */

#include "config.h"

extern int globtimeout, globtimeoutlen, debug;
/* all tcp functions to be moved/recoded into here */

/* All the variables needed to be passed along from start to
   end of the test */
struct tcpdata {
	int filedes;
	time_t start;
	};

void	start_test_tcp(struct monitorent *here)
{
	/* set up the check, start to connect() to the remote host
	   then let service_test_tcp() watch for the fd to be
	   writable */

	struct tcpdata *localstruct = NULL;
	struct my_hostent *hp = NULL;
        struct sockaddr_in name;
	int errcode = 0;
	
	/* Allocate our memory */
        here->monitordata = MALLOC(sizeof(struct tcpdata));

	localstruct = here->monitordata;

	time(&localstruct->start);

	localstruct->filedes = open_sock();

	if (localstruct->filedes == -1)
	{
		/* Bad error, code around it later */
		abort();
	}

	hp = my_gethostbyname(here->checkent->hostname);

        /* zero out the space */
        memset ( &name, 0, sizeof ( name ) );

        /* copy data */
        memcpy((char*)&name.sin_addr, (char*)hp->my_h_addr, hp->h_length);

        /* set family type */
        name.sin_family = AF_INET;

        /* set the port we're connecting to */
        name.sin_port = htons(here->checkent->port);

	if (debug)
		printf("start_test_tcp() doing connect() to %s:%d\n", 
			here->checkent->hostname, here->checkent->port);

        errcode = connect(localstruct->filedes, (struct sockaddr*)&name,
                sizeof(struct sockaddr_in));

        if ((errcode < 0) && (errno != EINPROGRESS))
        {
                if (errno == ECONNREFUSED || errno == EINTR)
                {
                        here->retval = SYSM_CONNREF;
                } else if (errno == ENETUNREACH) {
                        here->retval = SYSM_NETUNRCH; /* Network Unreachable */
                } else if (errno==EHOSTDOWN || errno==EHOSTUNREACH) {
                        here->retval = SYSM_HOSTDOWN; /* Host down */
                } else if (errno == ETIMEDOUT) {
                        here->retval = SYSM_TIMEDOUT; /* Conn timed out */
                }
		close(localstruct->filedes);

                /* Free memory we'd normally leak */
                FREE(localstruct);
                here->monitordata = NULL;

                return;
        } 

	/* poll it again later */
	return;

}

void
service_test_tcp(struct monitorent *here)
{
	/* take the check started by start_test_tcp() and watch our
	   fd for can_write() status, so we can close it down, and
	   let folks know that the port is answering.  Timeout is
	   either the kernel timeout, or 30 seconds from test start
	   time.  This time can be reconfigured */

	struct tcpdata *localstruct = NULL;
	int isopenretval = -1;

	if (debug)
		printf("doing service_test_tcp() of %s:%d\n",
			here->checkent->hostname, here->checkent->port);

	localstruct = here->monitordata;
	if (localstruct == NULL)
	{
		printf("bug - localstruct == NULL in tcp.c:service_test_tcp\n");
		return;
	}

	isopenretval = is_open(localstruct->filedes);
	if ((isopenretval != -1) && (isopenretval != SYSM_INPROG))
	{
		if (debug)
			printf("is_open() in srv_test_tcp() returned true\n");
		here->retval = isopenretval;
		close(localstruct->filedes);
		FREE(localstruct);
		here->monitordata = NULL;
		if (debug)
			printf("ending service_test_tcp() with retval = %s\n",
				errtostr(here->retval));
		return;
	}

        time(&localstruct->start);

	if ((time(NULL)-localstruct->start) >= globtimeout) 
	{
		here->retval = SYSM_TIMEDOUT;
		close(localstruct->filedes);
                FREE(localstruct);
                here->monitordata = NULL;
                if (debug)
			printf("ending service_test_tcp() with retval = %s\n",
				errtostr(here->retval));
                return;
	}

	return;
}

void
stop_test_tcp(struct monitorent *here)
{
        struct tcpdata *localstruct = NULL;
	
	localstruct = here->monitordata;

	close(localstruct->filedes);
	FREE(localstruct);
	here->monitordata = NULL;
        return;
}

