/* $Id: udp.c,v 1.25 1997/12/09 15:54:30 jared Exp $ */
#include "config.h"

extern int debug, globtimeout;

/* all udp functions to be moved/recoded into here */

#define UDP_CONNECT	1 /* doing connect() */
#define UDP_SENT_PKT	2 /* sent a packet */
#define UDP_REXMIT	3 /* re-xmited pkt */
#define UDP_GOTRESP	4 /* got a response */

struct udpdata {
	int filedes;
	unsigned char state;
	time_t start;
	time_t lastsent;
	unsigned int tries;
	};

void	start_test_udp(struct monitorent *here)
{
	/* start a  test of an udp port on a remote server. do the
	   connect() part, and everything else so that service_test_udp
	   can find out if the port is working.  udp tests tend to be a
	   bit broken, as not all servers respond to bogus packets, and
	   since this is generic, it's hard to tell what will happen.  It
	   may crash some software */
	   

	struct udpdata *localstruct = NULL;
	struct my_hostent *hp = NULL;
        struct sockaddr_in name;
	int errcode = 0;
	
	/* Allocate our memory */
        here->monitordata = MALLOC(sizeof(struct udpdata));

	localstruct = here->monitordata;

	memset(localstruct, 0, sizeof(localstruct));

	time(&localstruct->start);
	localstruct->lastsent=localstruct->start;
	localstruct->tries = 0;
	localstruct->filedes = udp_open_sock();

	if (localstruct->filedes == -1)
	{
		/* Bad error, code around it later */
		abort();
	}

	hp = my_gethostbyname(here->checkent->hostname);

        /* zero out the space */
        memset ( &name, 0, sizeof ( name ) );

        /* copy data */
        memcpy((char*)&name.sin_addr, (char*)hp->my_h_addr, hp->h_length);

        /* set family type */
        name.sin_family = AF_INET;

        /* set the port we're connecting to */
        name.sin_port = htons(here->checkent->port);

	if (debug)
		fprintf(stderr, "start_test_udp() doing connect() to %s:%d\n", 
			here->checkent->hostname, here->checkent->port);

        errcode = connect(localstruct->filedes, (struct sockaddr*)&name,
                sizeof(struct sockaddr_in));

        time(&localstruct->start);

        if ((errcode < 0) && (errno != EINPROGRESS))
        {
                if (close(localstruct->filedes) == -1)
                        abort(); /* close the fd incase it's left */ 
		switch(errno)
		{
			case ECONNREFUSED:
			case EINTR:
				here->retval = SYSM_CONNREF;
				break;
			case ENETUNREACH:
				here->retval = SYSM_NETUNRCH;
				break;
			case EHOSTDOWN:
			case EHOSTUNREACH:
                        	here->retval = SYSM_HOSTDOWN;
				break;
			case ETIMEDOUT:
				here->retval = SYSM_TIMEDOUT;
				break;
		}

                /* Free memory we'd normally leak */
                FREE(localstruct);
                here->monitordata = NULL;

                return;
        } 

	/* test set up -- check it again later */
	return;
}

void	service_test_udp(struct monitorent *here)
{
	/* take the socket created in start_test_udp() and figure out
	   if connect() returned errors, and if we can send data to that
	   port and get a response, etc..
	 */

	struct udpdata *localstruct = NULL;
        char buf[100];         /* buffer */
	int errcode=0;
	int serrno=0;

	if (debug)
		fprintf(stderr, "doing service_test_udp() of %s:%d\n",
			here->checkent->hostname, here->checkent->port);

	localstruct = here->monitordata;
	
	if (localstruct == NULL)
	{
		printf("bug! localstruct == NULL in udp.c:service_test_udp\n");
		return;
	}

	if (debug)
	{
		fprintf(stderr, "service_test_tcp() now-lastsent = %d\n",
			(int)(time(NULL)-localstruct->lastsent));
	}
	if ((time(NULL)-localstruct->lastsent) <1)
	{
		return;
	}

	memset(buf, 0x33, sizeof(buf));

	if (data_waiting_read(localstruct->filedes, 0))
	{
		if (debug)
			fprintf(stderr, "service_test_tcp() calling recv()\n");
		errcode = recv(localstruct->filedes, buf, sizeof(buf)-1, 0);
		if (debug)
			fprintf(stderr, "recv() returned %d\n", errcode);
		if (errcode == -1)
		{
			serrno=errno;
			if (debug)
			{
				perror("udp.c:recv");
			}
		}
		/* if we received a response in any way shape or form */
		if (errcode > 0)
		{
			here->retval = SYSM_OK;
	                close(localstruct->filedes);
	                FREE(localstruct);
	                here->monitordata = NULL;
	                if (debug)
	                        printf("ending service_test_udp() with retval = %s\n",
	                                errtostr(here->retval));
	                return;
		}
	} else {
		errcode = send (localstruct->filedes, buf, sizeof(buf), 0);
		if (errcode == -1)
		{
			fprintf(stderr, "send returned %d\n", errcode);
			serrno=errno;
		}
                if (debug)
                {
			fprintf(stderr, "service_test_udp() just sent %d\n", sizeof(buf));
                }

                time(&localstruct->lastsent);

		localstruct->tries++;

		if (localstruct->tries >= MAX_TRIES)
		{
			here->retval = SYSM_TIMEDOUT;
			close(localstruct->filedes);
			FREE(localstruct);
			here->monitordata = NULL;
			if (debug)
				fprintf(stderr, "ending service_test_udp() with retval = %s\n", errtostr(here->retval));
			return;
		}
	}
	if (serrno != 0)
	{
		/*
		** This is fucking digusting.  This really needs to go into
		** a seperate function.  I've seen this about 20 times now.
		** --Majdi
		*/

		switch(serrno)
		{
			case ECONNREFUSED:
			case EINTR:
                        	here->retval = SYSM_CONNREF;
				break;
			case ENETUNREACH:
				here->retval = SYSM_NETUNRCH;
				break;
			case EHOSTDOWN:
			case EHOSTUNREACH:
                        	here->retval = SYSM_HOSTDOWN;
				break;
			case ETIMEDOUT:
				here->retval = SYSM_TIMEDOUT;
				break;
			case EINPROGRESS:
				here->retval = SYSM_INPROG;
				break;
			case EBADF:
				here->retval = -1;
                }

		if (here->retval != 0)
		{
			close(localstruct->filedes);
			FREE(localstruct);
			here->monitordata = NULL;
			if (debug)
				fprintf(stderr, "ending service_test_udp() with retval = %s\n",
					errtostr(here->retval));
		return;
		}
	}

	/* a timeout possibility */
	if ((time(NULL)-localstruct->start) >= globtimeout) 
	{
		here->retval = SYSM_TIMEDOUT;
		close(localstruct->filedes);
                FREE(localstruct);
                here->monitordata = NULL;
                if (debug)
			fprintf(stderr, "ending service_test_udp() with retval = %s\n",
				errtostr(here->retval));
                return;
	}
	return;
}

int	udp_open_sock(int sock)
{
	/* Open a UDP network connection through the named socket */
	sock = socket(AF_INET, SOCK_DGRAM, 0);
	if (sock == -1)
	{
		if (debug)
			perror("udp_open_sock: opening socket");
		exit(2);
	}
	return sock;
}


void	stop_test_udp(struct monitorent *here)
{
        struct udpdata *localstruct = NULL;

        localstruct = here->monitordata;

        close(localstruct->filedes);
        FREE(localstruct);
        here->monitordata = NULL;
        return;
}

